@@ -1,4 +1,4 @@
-/* $Id: rmagick.h,v 1.110 2006/06/02 23:26:53 rmagick Exp $ */
+/* $Id: rmagick.h,v 1.152.2.4 2007/06/09 16:44:30 rmagick Exp $ */
 /*=============================================================================
 |               Copyright (C) 2006 by Timothy P. Hunter
 | Name:     rmagick.h
@@ -27,16 +27,18 @@
 #undef PACKAGE_STRING
 #undef PACKAGE_BUGREPORT
 #undef PACKAGE_TARNAME
+
+
 #include "magick/api.h"
 
-
-
 // Undef ImageMagick's versions of these symbols
 #undef PACKAGE_VERSION
 #undef PACKAGE_NAME
 #undef PACKAGE_STRING
 #undef PACKAGE_BUGREPORT
 #undef PACKAGE_TARNAME
+
+
 #include "rmagick_config.h"
 
 // Define a pair of macros that make it easier to code
@@ -74,9 +76,13 @@
 #undef DegreesToRadians     // defined in ImageMagick.h in 6.0.2
 #define DegreesToRadians(x) ((x)*3.14159265358979323846/180.0)
 
+#define PIXEL_INTENSITY(q) \
+        ((Quantum)(0.299*(q)->red + 0.587*(q)->green + 0.114*(q)->blue + 0.5))
+
 #define LMAX(a,b) ((((long)(a))>((long)(b)))?((long)(a)):((long)(b)))
 #define FMAX(a,b) ((((double)(a))>((double)(b)))?((double)(a)):((double)(b)))
 #define FMIN(a,b) ((((double)(a))<=((double)(b)))?((double)(a)):((double)(b)))
+#define RMAGICK_PI 3.14159265358979
 
 typedef ImageInfo Info; // Make type name match class name
 typedef PixelPacket Pixel;
@@ -97,9 +103,10 @@ struct TmpFile_Name
 
 typedef struct
 {
-    DrawInfo *info;         // the DrawInfo struct
-    VALUE primitives;       // the primitive string
+    DrawInfo *info;             // the DrawInfo struct
+    VALUE primitives;           // the primitive string
     struct TmpFile_Name *tmpfile_ary;
+    PixelPacket shadow_color;   // PolaroidOptions#shadow_color
 } Draw;             // make the type match the class name
 
 // Enum
@@ -115,7 +122,7 @@ typedef enum
 {
     False = 0,
     True = 1
-} boolean;
+} rm_boolean;
 
 typedef enum {
     AnyWeight,
@@ -163,6 +170,8 @@ typedef enum _QuantumExpressionOperator
     AndQuantumOperator,
     DivideQuantumOperator,
     LShiftQuantumOperator,
+    MaxQuantumOperator,
+    MinQuantumOperator,
     MultiplyQuantumOperator,
     OrQuantumOperator,
     RShiftQuantumOperator,
@@ -202,14 +211,46 @@ typedef enum _QuantumExpressionOperator
     #endif
 #endif
 
+// IM < 6.2.0 and GM don't have MagickBooleanType
 #if !defined(HAVE_MAGICKBOOLEANTYPE)
 typedef unsigned int MagickBooleanType;
+#if !defined(MagickTrue)
+#define MagickTrue 1
+#endif
+#if !defined(MagickFalse)
+#define MagickFalse 0
+#endif
 #endif
 
 #if !defined(HAVE_UNDEFINEDGRAVITY)
 #define UndefinedGravity 0
 #endif
 
+#if !defined(HAVE_QUANTUMPIXEL)
+#if QuantumDepth == 8
+#define QuantumPixel CharPixel
+#else
+#define QuantumPixel ShortPixel
+#endif
+#endif
+
+#if !defined(HAVE_SETIMAGEINFOFILE)
+#define SetImageInfoFile(info, fptr) (info)->file = (fptr)
+#endif
+
+#define ROUND_TO_QUANTUM(value) ((Quantum) ((value) > MaxRGB ? MaxRGB : (value) + 0.5))
+
+#if !defined(HAVE_SETIMAGEEXTENT)
+#define SetImageExtent(img, c, r) \
+    (img)->columns = (c); \
+    (img)->rows = (r)
+#endif
+
+
+#if !defined(HAVE_SETIMAGEBACKGROUNDCOLOR)
+#define SetImageBackgroundColor(img) SetImage((img), OpaqueOpacity)
+#endif
+
 // This implements the "omitted storage class model" for external variables.
 // (Ref: Harbison & Steele.) The rmmain.c file defines MAIN, which causes
 // the single defining declarations to be generated. No other source files
@@ -228,6 +269,7 @@ EXTERN VALUE Module_Magick;
 EXTERN VALUE Class_ImageList;
 EXTERN VALUE Class_Info;
 EXTERN VALUE Class_Draw;
+EXTERN VALUE Class_DrawOptions;
 EXTERN VALUE Class_Image;
 EXTERN VALUE Class_Montage;
 EXTERN VALUE Class_ImageMagickError;
@@ -241,6 +283,7 @@ EXTERN VALUE Class_Geometry;
 EXTERN VALUE Class_GeometryValue;   // Defined in RMagick.rb
 EXTERN VALUE Class_Pixel;
 EXTERN VALUE Class_Point;
+EXTERN VALUE Class_PolaroidOptions;
 EXTERN VALUE Class_Primary;
 EXTERN VALUE Class_Rectangle;
 EXTERN VALUE Class_Segment;
@@ -273,6 +316,9 @@ EXTERN VALUE Class_FilterTypes;
 EXTERN VALUE Class_GravityType;
 EXTERN VALUE Class_ImageType;
 EXTERN VALUE Class_InterlaceType;
+#if defined(HAVE_INTERPOLATEPIXELCOLOR)
+EXTERN VALUE Class_InterpolatePixelMethod;
+#endif
 #if defined(HAVE_COMPAREIMAGELAYERS)
 EXTERN VALUE Class_MagickLayerMethod;
 #endif
@@ -293,29 +339,30 @@ EXTERN VALUE Class_VirtualPixelMethod;
 /*
 *   Commonly-used IDs
 */
-EXTERN ID ID__dummy_img_;       // "_dummy_img_"
-EXTERN ID ID_call;              // "call"
-EXTERN ID ID_changed;           // "changed"
-EXTERN ID ID_cur_image;         // "cur_image"
-EXTERN ID ID_dup;               // "dup"
-EXTERN ID ID_enumerators;       // "enumerators"
-EXTERN ID ID_fill;              // "fill"
-EXTERN ID ID_flag;              // "flag"
-EXTERN ID ID_from_s;            // "from_s"
-EXTERN ID ID_Geometry;          // "Geometry"
-EXTERN ID ID_GeometryValue;     // "GeometryValue"
-EXTERN ID ID_height;            // "height"
-EXTERN ID ID_initialize_copy;   // "initialize_copy"
-EXTERN ID ID_length;            // "length"
-EXTERN ID ID_notify_observers;  // "notify_observers"
-EXTERN ID ID_new;               // "new"
-EXTERN ID ID_push;              // "push"
-EXTERN ID ID_spaceship;         // "<=>
-EXTERN ID ID_to_s;              // "to_s"
-EXTERN ID ID_values;            // "values"
-EXTERN ID ID_width;             // "width"
-EXTERN ID ID_x;                 // "x"
-EXTERN ID ID_y;                 // "y"
+EXTERN ID rm_ID__dummy_img_;       // "_dummy_img_"
+EXTERN ID rm_ID_call;              // "call"
+EXTERN ID rm_ID_changed;           // "changed"
+EXTERN ID rm_ID_cur_image;         // "cur_image"
+EXTERN ID rm_ID_dup;               // "dup"
+EXTERN ID rm_ID_enumerators;       // "enumerators"
+EXTERN ID rm_ID_fill;              // "fill"
+EXTERN ID rm_ID_flag;              // "flag"
+EXTERN ID rm_ID_from_s;            // "from_s"
+EXTERN ID rm_ID_Geometry;          // "Geometry"
+EXTERN ID rm_ID_GeometryValue;     // "GeometryValue"
+EXTERN ID rm_ID_height;            // "height"
+EXTERN ID rm_ID_initialize_copy;   // "initialize_copy"
+EXTERN ID rm_ID_length;            // "length"
+EXTERN ID rm_ID_notify_observers;  // "notify_observers"
+EXTERN ID rm_ID_new;               // "new"
+EXTERN ID rm_ID_push;              // "push"
+EXTERN ID rm_ID_spaceship;         // "<=>
+EXTERN ID rm_ID__tmpnam_;          // "_tmpnam"
+EXTERN ID rm_ID_to_s;              // "to_s"
+EXTERN ID rm_ID_values;            // "values"
+EXTERN ID rm_ID_width;             // "width"
+EXTERN ID rm_ID_x;                 // "x"
+EXTERN ID rm_ID_y;                 // "y"
 
 
 #define min(a,b) ((a)<(b)?(a):(b))
@@ -351,6 +398,9 @@ EXTERN ID ID_y;                 // "y"
 #define DCL_ATTR_ACCESSOR(class, attr) \
     DCL_ATTR_READER(class, attr) \
     DCL_ATTR_WRITER(class, attr)
+// Borrow another class' attribute writer definition
+#define SHARE_ATTR_WRITER(to, from, attr) \
+    rb_define_method(Class_##to, #attr "=", from##_##attr##_eq, 1);
 
 /*
     Define simple attribute accessor methods (boolean, int, string, and double types)
@@ -438,9 +488,9 @@ Pixel_##_channel_##_eq(VALUE self, VALUE v) \
     rm_check_frozen(self); \
     Data_Get_Struct(self, Pixel, pixel); \
     pixel->_channel_ = (Quantum) NUM2UINT(v); \
-    rb_funcall(self, ID_changed, 0); \
-    rb_funcall(self, ID_notify_observers, 1, self); \
-    return INT2NUM(pixel->_channel_); \
+    (void) rb_funcall(self, rm_ID_changed, 0); \
+    (void) rb_funcall(self, rm_ID_notify_observers, 1, self); \
+    return UINT2NUM((unsigned int)(pixel->_channel_)); \
 }
 
 
@@ -456,9 +506,9 @@ Pixel_##_cmyk_channel_##_eq(VALUE self, VALUE v) \
     rm_check_frozen(self); \
     Data_Get_Struct(self, Pixel, pixel); \
     pixel->_rgb_channel_ = (Quantum) NUM2UINT(v); \
-    rb_funcall(self, ID_changed, 0); \
-    rb_funcall(self, ID_notify_observers, 1, self); \
-    return INT2NUM(pixel->_rgb_channel_); \
+    (void) rb_funcall(self, rm_ID_changed, 0); \
+    (void) rb_funcall(self, rm_ID_notify_observers, 1, self); \
+    return UINT2NUM((unsigned int)pixel->_rgb_channel_); \
 } \
  \
 extern VALUE \
@@ -529,6 +579,7 @@ ATTR_WRITER(Draw, rotation)
 ATTR_WRITER(Draw, stroke)
 ATTR_WRITER(Draw, stroke_width)
 ATTR_WRITER(Draw, text_antialias)
+ATTR_WRITER(Draw, tile)
 ATTR_WRITER(Draw, undercolor)
 extern VALUE Draw_annotate(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
 extern VALUE Draw_clone(VALUE);
@@ -542,10 +593,14 @@ extern VALUE Draw_initialize(VALUE);
 extern VALUE Draw_inspect(VALUE);
 #if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
 extern VALUE Draw_alloc(VALUE);
+extern VALUE DrawOptions_alloc(VALUE);
 #else
 extern VALUE Draw_new(VALUE);
+extern VALUE DrawOptions_new(VALUE);
 #endif
 extern VALUE Draw_primitive(VALUE, VALUE);
+extern VALUE DrawOptions_initialize(VALUE);
+
 
 ATTR_WRITER(Montage, background_color)
 ATTR_WRITER(Montage, border_color)
@@ -573,6 +628,15 @@ extern VALUE Montage_new(VALUE);
 extern VALUE rm_montage_new(void);
 
 
+extern VALUE PolaroidOptions_alloc(VALUE);
+#if !defined(HAVE_RB_DEFINE_ALLOC_FUNC)
+extern VALUE PolaroidOptions_new(VALUE);
+#endif
+extern VALUE PolaroidOptions_initialize(VALUE);
+extern VALUE rm_polaroid_new(void);
+ATTR_WRITER(PolaroidOptions, shadow_color);
+
+
 // rmmain.c
 extern VALUE rm_montage_new(void);
 
@@ -585,7 +649,8 @@ extern VALUE ImageList_coalesce(VALUE);
 extern VALUE ImageList_deconstruct(VALUE);
 extern VALUE ImageList_display(VALUE);
 extern VALUE ImageList_flatten_images(VALUE);
-extern VALUE ImageList_map(VALUE, VALUE, VALUE);
+extern VALUE ImageList_fx(int, VALUE *, VALUE);
+extern VALUE ImageList_map(int, VALUE *, VALUE);
 extern VALUE ImageList_montage(VALUE);
 extern VALUE ImageList_morph(VALUE, VALUE);
 extern VALUE ImageList_mosaic(VALUE);
@@ -631,6 +696,7 @@ ATTR_WRITER(Info, monitor)
 ATTR_ACCESSOR(Info, monochrome)
 ATTR_ACCESSOR(Info, number_scenes)
 ATTR_ACCESSOR(Info, orientation)
+ATTR_ACCESSOR(Info, origin)
 ATTR_ACCESSOR(Info, page)
 ATTR_ACCESSOR(Info, pen)
 // ATTR_ACCESSOR(Info, ping) obsolete
@@ -641,6 +707,7 @@ ATTR_ACCESSOR(Info, scene)
 ATTR_ACCESSOR(Info, server_name)
 ATTR_ACCESSOR(Info, subimage)
 ATTR_ACCESSOR(Info, subrange)
+ATTR_WRITER(Info, texture)
 ATTR_ACCESSOR(Info, tile)
 ATTR_ACCESSOR(Info, size)
 ATTR_ACCESSOR(Info, units)
@@ -656,6 +723,7 @@ extern VALUE Info_new(VALUE);
 extern VALUE Info_define(int, VALUE *, VALUE);
 extern VALUE Info_aset(VALUE, VALUE, VALUE, VALUE);
 extern VALUE Info_aref(VALUE, VALUE, VALUE);
+extern VALUE Info_channel(int, VALUE *, VALUE);
 extern VALUE Info_undefine(VALUE, VALUE, VALUE);
 extern VALUE Info_initialize(VALUE);
 extern VALUE rm_info_new(void);
@@ -666,12 +734,13 @@ ATTR_ACCESSOR(Image, background_color)
 ATTR_READER(Image, base_columns)
 ATTR_READER(Image, base_filename)
 ATTR_READER(Image, base_rows)
+ATTR_ACCESSOR(Image, bias)
+ATTR_ACCESSOR(Image, black_point_compensation)
 ATTR_ACCESSOR(Image, blur)
 ATTR_ACCESSOR(Image, border_color)
 ATTR_READER(Image, bounding_box)
 ATTR_ACCESSOR(Image, chromaticity)
 ATTR_ACCESSOR(Image, class_type)
-ATTR_WRITER(Image, clip_mask)
 ATTR_ACCESSOR(Image, color_profile)
 ATTR_READER(Image, colors)
 ATTR_ACCESSOR(Image, colorspace)
@@ -696,6 +765,7 @@ ATTR_ACCESSOR(Image, image_type)
 ATTR_ACCESSOR(Image, interlace)
 ATTR_ACCESSOR(Image, iptc_profile)
 ATTR_ACCESSOR(Image, iterations)
+ATTR_ACCESSOR(Image, mask)
 ATTR_ACCESSOR(Image, matte)
 ATTR_ACCESSOR(Image, matte_color)
 ATTR_READER(Image, mean_error_per_pixel)
@@ -707,8 +777,9 @@ ATTR_READER(Image, normalized_maximum_error)
 ATTR_READER(Image, number_colors)
 ATTR_ACCESSOR(Image, offset)
 ATTR_WRITER(Image, opacity)
-ATTR_READER(Image, orientation)
+ATTR_ACCESSOR(Image, orientation)
 ATTR_ACCESSOR(Image, page)
+ATTR_ACCESSOR(Image, pixel_interpolation_method)
 ATTR_READER(Image, quality)
 ATTR_READER(Image, quantum_depth)
 ATTR_ACCESSOR(Image, rendering_intent)
@@ -718,11 +789,15 @@ ATTR_ACCESSOR(Image, start_loop)
 ATTR_ACCESSOR(Image, ticks_per_second)
 ATTR_ACCESSOR(Image, tile_info)
 ATTR_READER(Image, total_colors)
+ATTR_ACCESSOR(Image, transparent_color)
 ATTR_ACCESSOR(Image, units)
 ATTR_ACCESSOR(Image, virtual_pixel_method)
 ATTR_ACCESSOR(Image, x_resolution)
 ATTR_ACCESSOR(Image, y_resolution)
 
+extern ChannelType extract_channels(int *, VALUE *);
+extern void raise_ChannelType_error(VALUE);
+
 #if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
 extern VALUE Image_alloc(VALUE);
 extern VALUE Image_initialize(int, VALUE *, VALUE);
@@ -731,15 +806,24 @@ extern VALUE Image_new(int, VALUE *, VALUE);
 extern VALUE Image_initialize(VALUE, VALUE, VALUE, VALUE, VALUE);
 #endif
 
+extern VALUE Image_adaptive_blur(int, VALUE *, VALUE);
+extern VALUE Image_adaptive_blur_channel(int, VALUE *, VALUE);
+extern VALUE Image_adaptive_resize(int, VALUE *, VALUE);
+extern VALUE Image_adaptive_sharpen(int, VALUE *, VALUE);
+extern VALUE Image_adaptive_sharpen_channel(int, VALUE *, VALUE);
 extern VALUE Image_adaptive_threshold(int, VALUE *, VALUE);
 extern VALUE Image_add_noise(VALUE, VALUE);
 extern VALUE Image_add_noise_channel(int, VALUE *, VALUE);
+extern VALUE Image_add_profile(VALUE, VALUE);
 extern VALUE Image_affine_transform(VALUE, VALUE);
 extern VALUE Image_aref(VALUE, VALUE);
 extern VALUE Image_aset(VALUE, VALUE, VALUE);
+extern VALUE Image_auto_orient(VALUE);
+extern VALUE Image_auto_orient_bang(VALUE);
 extern VALUE Image_properties(VALUE);
 extern VALUE Image_bilevel_channel(int, VALUE *, VALUE);
 extern VALUE Image_black_threshold(int, VALUE *, VALUE);
+extern VALUE Image_blend(int, VALUE *, VALUE);
 extern VALUE Image_blur_image(int, VALUE *, VALUE);
 extern VALUE Image_blur_channel(int, VALUE *, VALUE);
 extern VALUE Image_border(VALUE, VALUE, VALUE, VALUE);
@@ -762,6 +846,8 @@ extern VALUE Image_colorize(int, VALUE *, VALUE);
 extern VALUE Image_colormap(int, VALUE *, VALUE);
 extern VALUE Image_composite(int, VALUE *, VALUE);
 extern VALUE Image_composite_bang(int, VALUE *, VALUE);
+extern VALUE Image_composite_channel(int, VALUE *, VALUE);
+extern VALUE Image_composite_channel_bang(int, VALUE *, VALUE);
 extern VALUE Image_composite_affine(VALUE, VALUE, VALUE);
 extern VALUE Image_compress_colormap_bang(VALUE);
 extern VALUE Image_constitute(VALUE, VALUE, VALUE, VALUE, VALUE);
@@ -773,10 +859,13 @@ extern VALUE Image_copy(VALUE);
 extern VALUE Image_crop(int, VALUE *, VALUE);
 extern VALUE Image_crop_bang(int, VALUE *, VALUE);
 extern VALUE Image_cycle_colormap(VALUE, VALUE);
+extern VALUE Image_delete_profile(VALUE, VALUE);
 extern VALUE Image_despeckle(VALUE);
 extern VALUE Image_difference(VALUE, VALUE);
 extern VALUE Image_dispatch(int, VALUE *, VALUE);
+extern VALUE Image_displace(int, VALUE *, VALUE);
 extern VALUE Image_display(VALUE);
+extern VALUE Image_dissolve(int, VALUE *, VALUE);
 extern VALUE Image_distortion_channel(int, VALUE *, VALUE);
 extern VALUE Image__dump(VALUE, VALUE);
 extern VALUE Image_dup(VALUE);
@@ -788,6 +877,7 @@ extern VALUE Image_equalize(VALUE);
 extern VALUE Image_erase_bang(VALUE);
 extern VALUE Image_export_pixels(int, VALUE *, VALUE);
 extern VALUE Image_export_pixels_to_str(int, VALUE *, VALUE);
+extern VALUE Image_find_similar_region(int, VALUE *, VALUE);
 extern VALUE Image_flip(VALUE);
 extern VALUE Image_flip_bang(VALUE);
 extern VALUE Image_flop(VALUE);
@@ -807,6 +897,7 @@ extern VALUE Image_init_copy(VALUE, VALUE);
 extern VALUE Image_inspect(VALUE);
 extern VALUE Image_level2(int, VALUE *, VALUE);
 extern VALUE Image_level_channel(int, VALUE *, VALUE);
+extern VALUE Image_linear_stretch(int, VALUE *, VALUE);
 extern VALUE Image__load(VALUE, VALUE);
 extern VALUE Image_magnify(VALUE);
 extern VALUE Image_magnify_bang(VALUE);
@@ -817,7 +908,7 @@ extern VALUE Image_minify(VALUE);
 extern VALUE Image_minify_bang(VALUE);
 extern VALUE Image_modulate(int, VALUE *, VALUE);
 extern VALUE Image_monochrome_q(VALUE);
-extern VALUE Image_motion_blur(VALUE, VALUE, VALUE, VALUE);
+extern VALUE Image_motion_blur(int, VALUE *, VALUE);
 extern VALUE Image_negate(int, VALUE *, VALUE);
 extern VALUE Image_negate_channel(int, VALUE *, VALUE);
 extern VALUE Image_normalize(VALUE);
@@ -829,7 +920,7 @@ extern VALUE Image_ordered_dither(int, VALUE *, VALUE);
 extern VALUE Image_palette_q(VALUE);
 extern VALUE Image_ping(VALUE, VALUE);
 extern VALUE Image_pixel_color(int, VALUE *, VALUE);
-// VALUE Image_plasma(VALUE, VALUE, VALUE, VALUE, VALUE, VALUE, VALUE);
+extern VALUE Image_polaroid(int, VALUE *, VALUE);
 extern VALUE Image_posterize(int, VALUE *, VALUE);
 extern VALUE Image_preview(VALUE, VALUE);
 extern VALUE Image_profile_bang(VALUE, VALUE, VALUE);
@@ -837,17 +928,19 @@ extern VALUE Image_quantize(int, VALUE *, VALUE);
 extern VALUE Image_quantization_error(VALUE);
 extern VALUE Image_quantum_operator(int, VALUE *, VALUE);
 extern VALUE Image_radial_blur(VALUE, VALUE);
+extern VALUE Image_radial_blur_channel(int, VALUE *, VALUE);
 extern VALUE Image_raise(int, VALUE *, VALUE);
 extern VALUE Image_random_channel_threshold(VALUE, VALUE, VALUE);
 extern VALUE Image_random_threshold_channel(int, VALUE *, VALUE);
 extern VALUE Image_read(VALUE, VALUE);
 extern VALUE Image_read_inline(VALUE, VALUE);
+extern VALUE Image_recolor(VALUE, VALUE);
 extern VALUE Image_reduce_noise(VALUE, VALUE);
 extern VALUE Image_resize(int, VALUE *, VALUE);
 extern VALUE Image_resize_bang(int, VALUE *, VALUE);
 extern VALUE Image_roll(VALUE, VALUE, VALUE);
-extern VALUE Image_rotate(VALUE, VALUE);
-extern VALUE Image_rotate_bang(VALUE, VALUE);
+extern VALUE Image_rotate(int, VALUE *, VALUE);
+extern VALUE Image_rotate_bang(int, VALUE *, VALUE);
 extern VALUE Image_sample(int, VALUE *, VALUE);
 extern VALUE Image_sample_bang(int, VALUE *, VALUE);
 extern VALUE Image_scale(int, VALUE *, VALUE);
@@ -864,6 +957,7 @@ extern VALUE Image_shave_bang(VALUE, VALUE, VALUE);
 extern VALUE Image_shear(VALUE, VALUE, VALUE);
 extern VALUE Image_sigmoidal_contrast_channel(int, VALUE *, VALUE);
 extern VALUE Image_signature(VALUE);
+extern VALUE Image_sketch(int, VALUE *, VALUE);
 extern VALUE Image_solarize(int, VALUE *, VALUE);
 extern VALUE Image_spaceship(VALUE, VALUE);
 extern VALUE Image_splice(int, VALUE *, VALUE);
@@ -882,12 +976,19 @@ extern VALUE Image_tint(int, VALUE *, VALUE);
 extern VALUE Image_to_blob(VALUE);
 extern VALUE Image_to_color(VALUE, VALUE);
 extern VALUE Image_transparent(int, VALUE *, VALUE);
+extern VALUE Image_transpose(VALUE);
+extern VALUE Image_transpose_bang(VALUE);
+extern VALUE Image_transverse(VALUE);
+extern VALUE Image_transverse_bang(VALUE);
 extern VALUE Image_trim(VALUE);
 extern VALUE Image_trim_bang(VALUE);
+extern VALUE Image_unique_colors(VALUE);
 extern VALUE Image_unsharp_mask(int, VALUE *, VALUE);
 extern VALUE Image_unsharp_mask_channel(int, VALUE *, VALUE);
 extern VALUE Image_vignette(int, VALUE *, VALUE);
+extern VALUE Image_watermark(int, VALUE *, VALUE);
 extern VALUE Image_wave(int, VALUE *, VALUE);
+extern VALUE Image_wet_floor(int, VALUE *, VALUE);
 extern VALUE Image_white_threshold(int, VALUE *, VALUE);
 extern VALUE Image_write(VALUE, VALUE);
 
@@ -949,9 +1050,11 @@ ATTR_ACCESSOR(Pixel, black)
 extern VALUE  Pixel_case_eq(VALUE, VALUE);
 extern VALUE  Pixel_clone(VALUE);
 extern VALUE  Pixel_dup(VALUE);
+extern VALUE  Pixel_eql_q(VALUE, VALUE);
 extern VALUE  Pixel_fcmp(int, VALUE *, VALUE);
 extern VALUE  Pixel_from_color(VALUE, VALUE);
 extern VALUE  Pixel_from_HSL(VALUE, VALUE);
+extern VALUE  Pixel_hash(VALUE);
 extern VALUE  Pixel_initialize(int, VALUE *, VALUE);
 extern VALUE  Pixel_init_copy(VALUE, VALUE);
 extern VALUE  Pixel_intensity(VALUE);
@@ -976,6 +1079,12 @@ extern VALUE  Segment_from_SegmentInfo(SegmentInfo *);
 extern void   AffineMatrix_to_AffineMatrix(AffineMatrix *, VALUE);
 extern void   ChromaticityInfo_to_ChromaticityInfo(ChromaticityInfo *, VALUE);
 extern void   Color_to_ColorInfo(ColorInfo *, VALUE);
+#if defined(HAVE_INTERPOLATEPIXELCOLOR)
+extern VALUE  InterpolatePixelMethod_new(InterpolatePixelMethod);
+#endif
+#if defined(HAVE_IMAGE_ORIENTATION)
+extern VALUE  OrientationType_new(OrientationType);
+#endif
 extern void   PrimaryInfo_to_PrimaryInfo(PrimaryInfo *, VALUE);
 extern void   Rectangle_to_RectangleInfo(RectangleInfo *, VALUE);
 extern void   Segment_to_SegmentInfo(SegmentInfo *, VALUE);
@@ -987,11 +1096,8 @@ extern VALUE  TypeMetric_from_TypeMetric(TypeMetric *);
 #if defined(HAVE_GETIMAGESTATISTICS)
 extern VALUE  Statistics_new(ImageStatistics *);
 #endif
-#if defined(HAVE_IMAGE_ORIENTATION)
-extern VALUE   OrientationType_new(OrientationType);
-#endif
 extern const char *StorageType_name(StorageType);
-extern VALUE   VirtualPixelMethod_new(VirtualPixelMethod);
+extern VALUE  VirtualPixelMethod_new(VirtualPixelMethod);
 
 #if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
 extern VALUE  Enum_alloc(VALUE);
@@ -1018,7 +1124,7 @@ extern char  *rm_string_value_ptr(volatile VALUE *);
 #endif
 extern char  *rm_string_value_ptr_len(volatile VALUE *, long *);
 extern int    rm_strcasecmp(const char *, const char *);
-extern void   rm_check_ary_len(VALUE, int);
+extern void   rm_check_ary_len(VALUE, long);
 extern void   rm_check_frozen(VALUE);
 extern int    rm_check_num2dbl(VALUE);
 extern double rm_fuzz_to_dbl(VALUE);
@@ -1031,6 +1137,7 @@ extern void   rm_not_implemented(void);
 extern void   rm_attr_write(VALUE, VALUE);
 extern void   rm_get_geometry(VALUE, long *, long *, unsigned long *, unsigned long *, int *);
 extern void   rm_split(Image *);
+extern void   rm_magick_error(const char *, const char *);
 
 typedef enum
 {
