@@ -1,6 +1,6 @@
-/* $Id: rmfill.c,v 1.14 2006/05/07 23:46:19 rmagick Exp $ */
+/* $Id: rmfill.c,v 1.17.2.1 2007/02/04 13:16:22 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2006 by Timothy P. Hunter
+|                Copyright (C) 2007 by Timothy P. Hunter
 | Name:     rmfill.c
 | Author:   Tim Hunter
 | Purpose:  GradientFill, TextureFill class definitions for RMagick
@@ -13,12 +13,12 @@ typedef struct
     double x1, y1, x2, y2;
     PixelPacket start_color;
     PixelPacket stop_color;
-} GradientFill;
+} rm_GradientFill;
 
 typedef struct
 {
     Image *texture;
-} TextureFill;
+} rm_TextureFill;
 
 /*
     Static:     free_Fill
@@ -45,7 +45,7 @@ GradientFill_new(
     VALUE start_color,
     VALUE stop_color)
 {
-    GradientFill *fill;
+    rm_GradientFill *fill;
     volatile VALUE new_fill;
     VALUE argv[6];
 
@@ -57,7 +57,7 @@ GradientFill_new(
     argv[5] = stop_color;
 
     new_fill = Data_Make_Struct(class
-                              , GradientFill
+                              , rm_GradientFill
                               , NULL
                               , free_Fill
                               , fill);
@@ -69,9 +69,9 @@ GradientFill_new(
 VALUE
 GradientFill_alloc(VALUE class)
 {
-    GradientFill *fill;
+    rm_GradientFill *fill;
 
-    return Data_Make_Struct(class, GradientFill, NULL, free_Fill, fill);
+    return Data_Make_Struct(class, rm_GradientFill, NULL, free_Fill, fill);
 }
 #endif
 
@@ -89,9 +89,9 @@ GradientFill_initialize(
     VALUE start_color,
     VALUE stop_color)
 {
-    GradientFill *fill;
+    rm_GradientFill *fill;
 
-    Data_Get_Struct(self, GradientFill, fill);
+    Data_Get_Struct(self, rm_GradientFill, fill);
 
     fill->x1 = NUM2DBL(x1);
     fill->y1 = NUM2DBL(y1);
@@ -116,7 +116,7 @@ point_fill(
     PixelPacket *stop_color)
 {
     double steps, distance;
-    long x, y;
+    unsigned long x, y;
     double red_step, green_step, blue_step;
 
     steps = sqrt((double)((image->columns-x0)*(image->columns-x0)
@@ -129,16 +129,16 @@ point_fill(
     {
         PixelPacket *row_pixels;
 
-        if (!(row_pixels = SetImagePixels(image, 0, y, image->columns, 1)))
+        if (!(row_pixels = SetImagePixels(image, 0, (long int)y, image->columns, 1)))
         {
             rm_check_image_exception(image, RetainOnError);
         }
         for (x = 0; x < image->columns; x++)
         {
             distance = sqrt((double)((x-x0)*(x-x0)+(y-y0)*(y-y0)));
-            row_pixels[x].red     = start_color->red   + (distance * red_step);
-            row_pixels[x].green   = start_color->green + (distance * green_step);
-            row_pixels[x].blue    = start_color->blue  + (distance * blue_step);
+            row_pixels[x].red     = ROUND_TO_QUANTUM(start_color->red   + (distance * red_step));
+            row_pixels[x].green   = ROUND_TO_QUANTUM(start_color->green + (distance * green_step));
+            row_pixels[x].blue    = ROUND_TO_QUANTUM(start_color->blue  + (distance * blue_step));
             row_pixels[x].opacity = OpaqueOpacity;
         }
         if (!SyncImagePixels(image))
@@ -161,7 +161,7 @@ vertical_fill(
     PixelPacket *stop_color)
 {
     double steps;
-    long x, y;
+    unsigned long x, y;
     volatile PixelPacket *master;
     double red_step, green_step, blue_step;
 
@@ -196,9 +196,9 @@ vertical_fill(
     for (x = 0; x < image->columns; x++)
     {
         double distance   = fabs(x1 - x);
-        master[x].red     = start_color->red   + (red_step * distance);
-        master[x].green   = start_color->green + (green_step * distance);
-        master[x].blue    = start_color->blue  + (blue_step * distance);
+        master[x].red     = ROUND_TO_QUANTUM(start_color->red   + (red_step * distance));
+        master[x].green   = ROUND_TO_QUANTUM(start_color->green + (green_step * distance));
+        master[x].blue    = ROUND_TO_QUANTUM(start_color->blue  + (blue_step * distance));
         master[x].opacity = OpaqueOpacity;
     }
 
@@ -207,7 +207,7 @@ vertical_fill(
     {
         PixelPacket *row_pixels;
 
-        if (!(row_pixels = SetImagePixels(image, 0, y, image->columns, 1)))
+        if (!(row_pixels = SetImagePixels(image, 0, (long int)y, image->columns, 1)))
         {
             rm_check_image_exception(image, RetainOnError);
         }
@@ -234,7 +234,7 @@ horizontal_fill(
     PixelPacket *stop_color)
 {
     double steps;
-    long x, y;
+    unsigned long x, y;
     volatile PixelPacket *master;
     double red_step, green_step, blue_step;
 
@@ -267,9 +267,9 @@ horizontal_fill(
     for (y = 0; y < image->rows; y++)
     {
         double distance   = fabs(y1 - y);
-        master[y].red     = start_color->red   + (distance * red_step);
-        master[y].green   = start_color->green + (distance * green_step);
-        master[y].blue    = start_color->blue  + (distance * blue_step);
+        master[y].red     = ROUND_TO_QUANTUM(start_color->red   + (distance * red_step));
+        master[y].green   = ROUND_TO_QUANTUM(start_color->green + (distance * green_step));
+        master[y].blue    = ROUND_TO_QUANTUM(start_color->blue  + (distance * blue_step));
         master[y].opacity = OpaqueOpacity;
     }
 
@@ -277,7 +277,7 @@ horizontal_fill(
     {
         PixelPacket *col_pixels;
 
-        if (!(col_pixels = SetImagePixels(image, x, 0, 1, image->rows)))
+        if (!(col_pixels = SetImagePixels(image, (long int)x, 0, 1, image->rows)))
         {
             rm_check_image_exception(image, RetainOnError);
         }
@@ -306,7 +306,7 @@ v_diagonal_fill(
     PixelPacket *start_color,
     PixelPacket *stop_color)
 {
-    long x, y;
+    unsigned long x, y;
     double red_step, green_step, blue_step;
     double m, b, steps = 0.0;
     double d1, d2;
@@ -325,7 +325,7 @@ v_diagonal_fill(
     {
         steps += FMAX(fabs(d1),fabs(d2));
     }
-    else if (d1 > image->rows && d2 > image->rows)
+    else if (d1 > (double)image->rows && d2 > (double)image->rows)
     {
         steps += FMAX(d1-image->rows, d2-image->rows);
     }
@@ -351,16 +351,16 @@ v_diagonal_fill(
     {
         PixelPacket *row_pixels;
 
-        if (!(row_pixels = SetImagePixels(image, 0, y, image->columns, 1)))
+        if (!(row_pixels = SetImagePixels(image, 0, (long int)y, image->columns, 1)))
         {
             rm_check_image_exception(image, RetainOnError);
         }
         for (x = 0; x < image->columns; x++)
         {
-            double distance = fabs(y-(m * x + b));
-            row_pixels[x].red     = start_color->red   + (distance * red_step);
-            row_pixels[x].green   = start_color->green + (distance * green_step);
-            row_pixels[x].blue    = start_color->blue  + (distance * blue_step);
+            double distance = (double) abs((int)(y-(m * x + b)));
+            row_pixels[x].red     = ROUND_TO_QUANTUM(start_color->red   + (distance * red_step));
+            row_pixels[x].green   = ROUND_TO_QUANTUM(start_color->green + (distance * green_step));
+            row_pixels[x].blue    = ROUND_TO_QUANTUM(start_color->blue  + (distance * blue_step));
             row_pixels[x].opacity = OpaqueOpacity;
         }
         if (!SyncImagePixels(image))
@@ -385,7 +385,7 @@ h_diagonal_fill(
     PixelPacket *start_color,
     PixelPacket *stop_color)
 {
-    long x, y;
+    unsigned long x, y;
     double m, b, steps = 0.0;
     double red_step, green_step, blue_step;
     double d1, d2;
@@ -398,7 +398,7 @@ h_diagonal_fill(
     // the left or right side of the image between y=0 and y=image->rows.
     // When y=0, x=-b/m. When y=image->rows, x = (image->rows-b)/m.
     d1 = -b/m;
-    d2 = (image->rows-b)/m;
+    d2 = (double) ((image->rows-b) / m);
 
     // If the line is entirely to the right or left of the image, increase
     // the number of steps.
@@ -406,9 +406,9 @@ h_diagonal_fill(
     {
         steps += FMAX(fabs(d1),fabs(d2));
     }
-    else if (d1 > image->columns && d2 > image->columns)
+    else if (d1 > (double)image->columns && d2 > (double)image->columns)
     {
-        steps += FMAX(fabs(image->columns-d1),fabs(image->columns-d2));
+        steps += FMAX(abs((int)(image->columns-d1)),abs((int)(image->columns-d2)));
     }
 
     d1 = FMAX(d1, image->columns-d1);
@@ -432,16 +432,16 @@ h_diagonal_fill(
     {
         PixelPacket *row_pixels;
 
-        if (!(row_pixels = SetImagePixels(image, 0, y, image->columns, 1)))
+        if (!(row_pixels = SetImagePixels(image, 0, (long int)y, image->columns, 1)))
         {
             rm_check_image_exception(image, RetainOnError);
         }
         for (x = 0; x < image->columns; x++)
         {
-            double distance = fabs(x-((y-b)/m));
-            row_pixels[x].red     = start_color->red   + (distance * red_step);
-            row_pixels[x].green   = start_color->green + (distance * green_step);
-            row_pixels[x].blue    = start_color->blue  + (distance * blue_step);
+            double distance = (double) abs((int)(x-((y-b)/m)));
+            row_pixels[x].red     = ROUND_TO_QUANTUM(start_color->red   + (distance * red_step));
+            row_pixels[x].green   = ROUND_TO_QUANTUM(start_color->green + (distance * green_step));
+            row_pixels[x].blue    = ROUND_TO_QUANTUM(start_color->blue  + (distance * blue_step));
             row_pixels[x].opacity = OpaqueOpacity;
         }
         if (!SyncImagePixels(image))
@@ -460,12 +460,12 @@ h_diagonal_fill(
 VALUE
 GradientFill_fill(VALUE self, VALUE image_obj)
 {
-    GradientFill *fill;
+    rm_GradientFill *fill;
     Image *image;
     PixelPacket start_color, stop_color;
     double x1, y1, x2, y2;          // points on the line
 
-    Data_Get_Struct(self, GradientFill, fill);
+    Data_Get_Struct(self, rm_GradientFill, fill);
     Data_Get_Struct(image_obj, Image, image);
 
     x1 = fill->x1;
@@ -527,9 +527,9 @@ GradientFill_fill(VALUE self, VALUE image_obj)
 static void
 free_TextureFill(void *fill_obj)
 {
-    TextureFill *fill = (TextureFill *)fill_obj;
+    rm_TextureFill *fill = (rm_TextureFill *)fill_obj;
 
-    DestroyImage(fill->texture);
+    (void) DestroyImage(fill->texture);
     xfree(fill);
 }
 
@@ -542,12 +542,12 @@ free_TextureFill(void *fill_obj)
 VALUE
 TextureFill_new(VALUE class, VALUE texture)
 {
-    TextureFill *fill;
+    rm_TextureFill *fill;
     VALUE argv[1];
     volatile VALUE new_fill;
 
     new_fill = Data_Make_Struct(class
-                              , TextureFill
+                              , rm_TextureFill
                               , NULL
                               , free_TextureFill
                               , fill);
@@ -559,9 +559,9 @@ TextureFill_new(VALUE class, VALUE texture)
 VALUE
 TextureFill_alloc(VALUE class)
 {
-    TextureFill *fill;
+    rm_TextureFill *fill;
     return Data_Make_Struct(class
-                        , TextureFill
+                        , rm_TextureFill
                         , NULL
                         , free_TextureFill
                         , fill);
@@ -575,17 +575,17 @@ TextureFill_alloc(VALUE class)
 VALUE
 TextureFill_initialize(VALUE self, VALUE texture_arg)
 {
-    TextureFill *fill;
+    rm_TextureFill *fill;
     Image *texture;
     volatile VALUE texture_image;
 
-    Data_Get_Struct(self, TextureFill, fill);
+    Data_Get_Struct(self, rm_TextureFill, fill);
 
     texture_image = ImageList_cur_image(texture_arg);
 
     // Bump the reference count on the texture image.
     Data_Get_Struct(texture_image, Image, texture);
-    ReferenceImage(texture);
+    (void) ReferenceImage(texture);
 
     fill->texture = texture;
     return self;
@@ -598,13 +598,13 @@ TextureFill_initialize(VALUE self, VALUE texture_arg)
 VALUE
 TextureFill_fill(VALUE self, VALUE image_obj)
 {
-    TextureFill *fill;
+    rm_TextureFill *fill;
     Image *image;
 
     Data_Get_Struct(image_obj, Image, image);
-    Data_Get_Struct(self, TextureFill, fill);
+    Data_Get_Struct(self, rm_TextureFill, fill);
 
-    TextureImage(image, fill->texture);
+    (void) TextureImage(image, fill->texture);
     rm_check_image_exception(image, RetainOnError);
 
     return self;
