@@ -1,6 +1,6 @@
-/* $Id: rmilist.c,v 1.40 2006/05/27 21:05:59 rmagick Exp $ */
+/* $Id: rmilist.c,v 1.46.2.1 2007/03/04 00:05:34 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2006 by Timothy P. Hunter
+|                Copyright (C) 2007 by Timothy P. Hunter
 | Name:     rmilist.c
 | Author:   Tim Hunter
 | Purpose:  ImageList class method definitions for RMagick
@@ -23,13 +23,14 @@ ImageList_animate(int argc, VALUE *argv, VALUE self)
     Info *info;
     volatile VALUE info_obj;
 
-    // Convert the images array to an images sequence.
-    images = rm_images_from_imagelist(self);
-
     if (argc > 1)
     {
         rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 or 1)", argc);
     }
+
+    // Convert the images array to an images sequence.
+    images = rm_images_from_imagelist(self);
+
     if (argc == 1)
     {
         Image *img;
@@ -77,7 +78,7 @@ ImageList_append(VALUE self, VALUE stack_arg)
     new_image = AppendImages(images, stack, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_image, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_image);
 
@@ -102,7 +103,7 @@ ImageList_average(VALUE self)
     new_image = AverageImages(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_image, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_image);
 
@@ -130,7 +131,7 @@ ImageList_coalesce(VALUE self)
     new_images = CoalesceImages(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
@@ -156,7 +157,7 @@ ImageList_deconstruct(VALUE self)
     new_images = DeconstructImages(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
@@ -174,13 +175,13 @@ ImageList_display(VALUE self)
     Info *info;
     volatile VALUE info_obj;
 
-    // Convert the images array to an images sequence.
-    images = rm_images_from_imagelist(self);
-
     // Create a new Info object to use with this call
     info_obj = rm_info_new();
     Data_Get_Struct(info_obj, Info, info);
 
+    // Convert the images array to an images sequence.
+    images = rm_images_from_imagelist(self);
+
     (void) DisplayImages(info, images);
     rm_split(images);
     rm_check_image_exception(images, RetainOnError);
@@ -205,7 +206,7 @@ ImageList_flatten_images(VALUE self)
     new_image = FlattenImages(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_image, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_image);
 
@@ -213,23 +214,77 @@ ImageList_flatten_images(VALUE self)
 }
 
 
+/*
+    Method:     ImageList#fx(expression[, channel...])
+*/
+VALUE
+ImageList_fx(int argc, VALUE *argv, VALUE self)
+{
+#if defined(HAVE_FXIMAGECHANNEL)
+    Image *images, *new_image;
+    char *expression;
+    ChannelType channels;
+    ExceptionInfo exception;
+
+
+    channels = extract_channels(&argc, argv);
+
+    // There must be exactly 1 remaining argument.
+    if (argc == 0)
+    {
+        rb_raise(rb_eArgError, "wrong number of arguments (0 for 1 or more)");
+    }
+    else if (argc > 1)
+    {
+        raise_ChannelType_error(argv[argc-1]);
+    }
+
+    expression = STRING_PTR(argv[0]);
+
+    images = rm_images_from_imagelist(self);
+    GetExceptionInfo(&exception);
+    new_image = FxImageChannel(images, channels, expression, &exception);
+    rm_split(images);
+    rm_check_exception(&exception, new_image, DestroyOnError);
+    (void) DestroyExceptionInfo(&exception);
+
+    rm_ensure_result(new_image);
+
+    return rm_image_new(new_image);
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
 
 /*
-    Method:     ImageList#map
+    Method:     ImageList#map(reference, dither=false)
     Purpose:    Call MapImages
     Returns:    a new ImageList with mapped images. @scene is set to self.scene
 */
 VALUE
-ImageList_map(VALUE self, VALUE map_image, VALUE dither_arg)
+ImageList_map(int argc, VALUE *argv, VALUE self)
 {
     Image *images, *new_images = NULL;
     Image *map;
-    unsigned int dither;
-    volatile VALUE image, scene, new_imagelist;
+    unsigned int dither = False;
+    volatile VALUE scene, new_imagelist;
     ExceptionInfo exception;
 
-    image = ImageList_cur_image(map_image);
-    Data_Get_Struct(image, Image, map);
+    switch (argc)
+    {
+        case 2:
+            dither = RTEST(argv[1]);
+        case 1:
+            Data_Get_Struct(ImageList_cur_image(argv[0]), Image, map);
+            break;
+        default:
+            rb_raise(rb_eArgError, "wrong number of arguments (%d for 1 or 2)", argc);
+            break;
+    }
+
 
     if (rm_imagelist_length(self) == 0)
     {
@@ -243,12 +298,11 @@ ImageList_map(VALUE self, VALUE map_image, VALUE dither_arg)
     new_images = CloneImageList(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
     // Call ImageMagick
-    dither = !(dither_arg == Qfalse || dither_arg == Qnil);
     (void) MapImages(new_images, map, dither);
     rm_check_image_exception(new_images, DestroyOnError);
 
@@ -280,7 +334,7 @@ ImageList_montage(VALUE self)
     {
         // Run the block in the instance's context, allowing the app to modify the
         // object's attributes.
-        rb_obj_instance_eval(0, NULL, montage_obj);
+        (void) rb_obj_instance_eval(0, NULL, montage_obj);
     }
 
     Data_Get_Struct(montage_obj, Montage, montage);
@@ -303,7 +357,7 @@ ImageList_montage(VALUE self)
     new_images = MontageImages(images, montage->info, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
@@ -337,12 +391,12 @@ ImageList_morph(VALUE self, VALUE nimages)
         rb_raise(rb_eArgError, "number of intervening images must be > 0");
     }
 
-    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
+    images = rm_images_from_imagelist(self);
     new_images = MorphImages(images, (unsigned long)number_images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
@@ -360,12 +414,12 @@ ImageList_mosaic(VALUE self)
     Image *images, *new_image;
     ExceptionInfo exception;
 
-    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
+    images = rm_images_from_imagelist(self);
     new_image = MosaicImages(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_image, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_image);
 
@@ -386,34 +440,39 @@ ImageList_optimize_layers(VALUE self, VALUE method)
     MagickLayerMethod mthd;
     ExceptionInfo exception;
 
-    images = rm_images_from_imagelist(self);
     GetExceptionInfo(&exception);
     VALUE_TO_ENUM(method, mthd, MagickLayerMethod);
+    images = rm_images_from_imagelist(self);
 
     switch (mthd)
     {
 #if defined(HAVE_COALESCELAYER)
-      case CoalesceLayer:
-        new_images = CoalesceImages(images, &exception);
-        break;
-      case DisposeLayer:
-        new_images = DisposeImages(images, &exception);
-        break;
+        case CoalesceLayer:
+            new_images = CoalesceImages(images, &exception);
+            break;
+        case DisposeLayer:
+            new_images = DisposeImages(images, &exception);
+            break;
 #endif
-      case OptimizeLayer:
-        new_images = OptimizeImageLayers(images, &exception);
-        break;
-      case OptimizePlusLayer:
-        new_images = OptimizePlusImageLayers(images, &exception);
-        break;
-      default:
-        new_images = CompareImageLayers(images, mthd, &exception);
-        break;
+        case OptimizeLayer:
+            new_images = OptimizeImageLayers(images, &exception);
+            break;
+        case OptimizePlusLayer:
+            new_images = OptimizePlusImageLayers(images, &exception);
+            break;
+        case CompareAnyLayer:
+        case CompareClearLayer:
+        case CompareOverlayLayer:
+            new_images = CompareImageLayers(images, mthd, &exception);
+            break;
+        default:
+            rb_raise(rb_eArgError, "undefined layer method");
+            break;
     }
 
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
@@ -434,7 +493,7 @@ ImageList_optimize_layers(VALUE self, VALUE method)
 VALUE
 rm_imagelist_new()
 {
-    return rb_funcall(Class_ImageList, ID_new, 0);
+    return rb_funcall(Class_ImageList, rm_ID_new, 0);
 }
 
 
@@ -463,7 +522,7 @@ rm_imagelist_from_images(Image *images)
           rm_imagelist_push(new_imagelist, rm_image_new(image));
      }
 
-     rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
+     (void) rb_iv_set(new_imagelist, "@scene", INT2FIX(0));
      return new_imagelist;
 }
 
@@ -507,7 +566,7 @@ VALUE
 rm_imagelist_scene_eq(VALUE imagelist, VALUE scene)
 {
      rm_check_frozen(imagelist);
-     rb_iv_set(imagelist, "@scene", scene);
+     (void) rb_iv_set(imagelist, "@scene", scene);
      return scene;
 }
 
@@ -520,7 +579,7 @@ rm_imagelist_length(VALUE imagelist)
 {
     volatile VALUE len;
 
-    len = rb_funcall(imagelist, ID_length, 0);
+    len = rb_funcall(imagelist, rm_ID_length, 0);
     return FIX2INT(len);
 }
 
@@ -532,7 +591,7 @@ void
 rm_imagelist_push(VALUE imagelist, VALUE image)
 {
     rm_check_frozen(imagelist);
-    (void) rb_funcall(imagelist, ID_push, 1, image);
+    (void) rb_funcall(imagelist, rm_ID_push, 1, image);
 }
 
 /*
@@ -556,15 +615,15 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
     switch (argc)
     {
         case 5:
-            quantize_info.measure_error = RTEST(argv[4]);
+            quantize_info.measure_error = (MagickBooleanType) RTEST(argv[4]);
         case 4:
-            quantize_info.tree_depth = NUM2INT(argv[3]);
+            quantize_info.tree_depth = (unsigned long)NUM2INT(argv[3]);
         case 3:
-            quantize_info.dither = RTEST(argv[2]);
+            quantize_info.dither = (MagickBooleanType) RTEST(argv[2]);
         case 2:
             VALUE_TO_ENUM(argv[1], quantize_info.colorspace, ColorspaceType);
         case 1:
-            quantize_info.number_colors = NUM2INT(argv[0]);
+            quantize_info.number_colors = NUM2ULONG(argv[0]);
         case 0:
             break;
         default:
@@ -583,12 +642,12 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
     new_images = CloneImageList(images, &exception);
     rm_split(images);
     rm_check_exception(&exception, new_images, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     rm_ensure_result(new_images);
 
 
-    QuantizeImages(&quantize_info, new_images);
+    (void) QuantizeImages(&quantize_info, new_images);
     rm_check_exception(&exception, new_images, DestroyOnError);
 
     // Create new ImageList object, convert mapped image sequence to images,
@@ -601,7 +660,7 @@ ImageList_quantize(int argc, VALUE *argv, VALUE self)
 
     // Set @scene in new ImageList object to same value as in self.
     scene = rb_iv_get(self, "@scene");
-    rb_iv_set(new_imagelist, "@scene", scene);
+    (void) rb_iv_set(new_imagelist, "@scene", scene);
 
     return new_imagelist;
 }
@@ -630,8 +689,8 @@ ImageList_to_blob(VALUE self)
     images = rm_images_from_imagelist(self);
 
     GetExceptionInfo(&exception);
-    (void) SetImageInfo(info, True, &exception);
-    CHECK_EXCEPTION()
+    (void) SetImageInfo(info, MagickTrue, &exception);
+    rm_check_exception(&exception, images, RetainOnError);
 
     if (*info->magick != '\0')
     {
@@ -645,7 +704,7 @@ ImageList_to_blob(VALUE self)
     // Unconditionally request multi-images support. The worst that
     // can happen is that there's only one image or the format
     // doesn't support multi-image files.
-    info->adjoin = True;
+    info->adjoin = MagickTrue;
 #if defined(HAVE_IMAGESTOBLOB)
     blob = ImagesToBlob(info, images, &length, &exception);
 #else
@@ -657,7 +716,7 @@ ImageList_to_blob(VALUE self)
     }
     rm_split(images);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
 
     if (length == 0 || !blob)
@@ -665,7 +724,7 @@ ImageList_to_blob(VALUE self)
         return Qnil;
     }
 
-    blob_str = rb_str_new(blob, length);
+    blob_str = rb_str_new(blob, (long)length);
     magick_free((void*)blob);
 
     return blob_str;
@@ -700,23 +759,20 @@ ImageList_write(VALUE self, VALUE file)
     volatile VALUE info_obj;
     char *filename;
     long filenameL;
-    int scene;
+    unsigned long scene;
     ExceptionInfo exception;
 
     info_obj = rm_info_new();
     Data_Get_Struct(info_obj, Info, info);
 
 
-    // Convert the images array to an images sequence.
-    images = rm_images_from_imagelist(self);
-
     if (TYPE(file) == T_FILE)
     {
         OpenFile *fptr;
 
         // Ensure file is open - raise error if not
         GetOpenFile(file, fptr);
-        info->file = GetReadFile(fptr);
+        SetImageInfoFile(info, GetReadFile(fptr));
     }
     else
     {
@@ -728,9 +784,12 @@ ImageList_write(VALUE self, VALUE file)
         filenameL = min(filenameL, MaxTextExtent-1);
         memcpy(info->filename, filename, (size_t)filenameL);
         info->filename[filenameL] = '\0';
-        info->file = NULL;
+        SetImageInfoFile(info, NULL);
     }
 
+    // Convert the images array to an images sequence.
+    images = rm_images_from_imagelist(self);
+
     // Copy the filename into each images. Set a scene number to be used if
     // writing multiple files. (Ref: ImageMagick's utilities/convert.c
     for (scene = 0, img = images; img; img = GetNextImageInList(img))
@@ -740,17 +799,20 @@ ImageList_write(VALUE self, VALUE file)
     }
 
     GetExceptionInfo(&exception);
-    (void) SetImageInfo(info, True, &exception);
+    (void) SetImageInfo(info, MagickTrue, &exception);
+    rm_check_exception(&exception, images, RetainOnError);
+    (void) DestroyExceptionInfo(&exception);
 
     // Find out if the format supports multi-images files.
+    GetExceptionInfo(&exception);
     m = GetMagickInfo(info->magick, &exception);
-    CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    rm_check_exception(&exception, images, RetainOnError);
+    (void) DestroyExceptionInfo(&exception);
 
     // Tell WriteImage if we want a multi-images file.
     if (rm_imagelist_length(self) > 1 && m->adjoin)
     {
-        info->adjoin = True;
+        info->adjoin = MagickTrue;
     }
 
     for (img = images; img; img = GetNextImageInList(img))
