@@ -1,6 +1,6 @@
-/* $Id: rminfo.c,v 1.28 2005/04/30 23:01:13 rmagick Exp $ */
+/* $Id: rminfo.c,v 1.45.2.2 2007/03/04 00:05:36 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2005 by Timothy P. Hunter
+|                Copyright (C) 2007 by Timothy P. Hunter
 | Name:     rminfo.c
 | Author:   Tim Hunter
 | Purpose:  Info class method definitions for RMagick.
@@ -8,6 +8,10 @@
 
 #include "rmagick.h"
 
+static VALUE get_option(VALUE, char *);
+static VALUE set_option(VALUE, char *, VALUE);
+
+
 DEF_ATTR_ACCESSOR(Info, antialias, bool)
 
 /*
@@ -88,7 +92,7 @@ Info_aset(VALUE self, VALUE format, VALUE key, VALUE value)
     key_p = STRING_PTR_LEN(key, key_l);
 
     /* Allow any argument that supports to_s */
-    value = rb_funcall(value, ID_to_s, 0);
+    value = rb_funcall(value, rm_ID_to_s, 0);
     value_p = STRING_PTR(value);
 
     if (format_l > MAX_FORMAT_LEN || format_l+key_l > MaxTextExtent-1)
@@ -121,7 +125,7 @@ Info_aset(VALUE self, VALUE format, VALUE key, VALUE value)
 
     format_p = STRING_PTR_LEN(format, format_l);
     key_p = STRING_PTR_LEN(key, key_l);
-    value = rb_funcall(value, ID_to_s, 0);
+    value = rb_funcall(value, rm_ID_to_s, 0);
     value_p = STRING_PTR_LEN(value, value_l);
 
     if ((3 + format_l + key_l + value_l) > sizeof(definitions))
@@ -136,7 +140,9 @@ Info_aset(VALUE self, VALUE format, VALUE key, VALUE value)
 
     GetExceptionInfo(&exception);
     okay = AddDefinitions(info, definitions, &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    (void) DestroyExceptionInfo(&exception);
+
     if (!okay)
     {
         rb_warn("%.60s:%.1024s not defined - AddDefinitions failed.", format_p, key_p);
@@ -152,6 +158,51 @@ Info_aset(VALUE self, VALUE format, VALUE key, VALUE value)
 }
 
 
+VALUE
+Info_authenticate(VALUE self)
+{
+    Info *info;
+
+    Data_Get_Struct(self, Info, info);
+    if (info->authenticate)
+    {
+        return rb_str_new2(info->authenticate);
+    }
+    else
+    {
+        return Qnil;
+    }
+}
+
+
+VALUE
+Info_authenticate_eq(VALUE self, VALUE passwd)
+{
+    Info *info;
+    char *passwd_p = NULL;
+    long passwd_len = 0;
+
+    Data_Get_Struct(self, Info, info);
+
+    if (!NIL_P(passwd))
+    {
+        passwd_p = STRING_PTR_LEN(passwd, passwd_len);
+    }
+
+    if (info->authenticate)
+    {
+        magick_free(info->authenticate);
+        info->authenticate = NULL;
+    }
+    if (passwd_len > 0)
+    {
+        magick_clone_string(&info->authenticate, passwd_p);
+    }
+
+    return self;
+}
+
+
 /*
     Method:     Info#background_color
     Purpose:    return the name of the background color as a String
@@ -210,6 +261,39 @@ Info_border_color_eq(VALUE self, VALUE bc_arg)
     return self;
 }
 
+
+
+/*
+    Method:     Info#channel(channel [, channel...])
+    Purpose:    Set the channels
+    Thanks:     Douglas Sellers
+*/
+VALUE
+Info_channel(int argc, VALUE *argv, VALUE self)
+{
+#if defined(HAVE_IMAGEINFO_CHANNEL)
+    Info *info;
+    ChannelType channels;
+
+    channels = extract_channels(&argc, argv);
+
+    // Ensure all arguments consumed.
+    if (argc > 0)
+    {
+        raise_ChannelType_error(argv[argc-1]);
+    }
+
+    Data_Get_Struct(self, Info, info);
+
+    info->channel = channels;
+    return self;
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
 /*
     Method:     Info#colorspace
     Purpose:    Get the colorspace type
@@ -238,6 +322,8 @@ Info_colorspace_eq(VALUE self, VALUE colorspace)
     return self;
 }
 
+OPTION_ATTR_ACCESSOR(comment, Comment)
+
 /*
     Method:  Info#compression
     Purpose: Get the compression type
@@ -290,7 +376,7 @@ Info_define(int argc, VALUE *argv, VALUE self)
     {
         case 3:
             /* Allow any argument that supports to_s */
-            fmt_arg = rb_funcall(argv[2], ID_to_s, 0);
+            fmt_arg = rb_funcall(argv[2], rm_ID_to_s, 0);
             value = STRING_PTR(fmt_arg);
         case 2:
             key = STRING_PTR_LEN(argv[1], key_l);
@@ -332,7 +418,7 @@ Info_define(int argc, VALUE *argv, VALUE self)
     {
         case 3:
             /* Allow any argument that supports to_s */
-            fmt_arg = rb_funcall(argv[2], ID_to_s, 0);
+            fmt_arg = rb_funcall(argv[2], rm_ID_to_s, 0);
             value = STRING_PTR_LEN(fmt_arg, value_l);
             /* Fall through */
         case 2:
@@ -357,7 +443,9 @@ Info_define(int argc, VALUE *argv, VALUE self)
 
     GetExceptionInfo(&exception);
     okay = AddDefinitions(info, definitions, &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    (void) DestroyExceptionInfo(&exception);
+
     if (!okay)
     {
         rb_warn("%.*s not defined - AddDefinitions failed.", sizeof(definitions), definitions);
@@ -372,6 +460,90 @@ Info_define(int argc, VALUE *argv, VALUE self)
 #endif
 }
 
+/*
+    Method:     Info#delay
+    Purpose:    Get the delay attribute
+    Notes:      Convert from string to numeric
+*/
+VALUE
+Info_delay(VALUE self)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    const char *delay;
+    char *p;
+    long d;
+
+    Data_Get_Struct(self, Info, info);
+
+    delay = GetImageOption(info, "delay");
+    if (delay)
+    {
+        d = strtol(delay, &p, 10);
+        if (*p != '\0')
+        {
+            rb_raise(rb_eRangeError, "failed to convert %s to Numeric", delay);
+        }
+        return LONG2NUM(d);
+    }
+    return Qnil;
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+/*
+ * Will raise an exception if `arg' can't be converted to an int.
+*/
+#if defined(HAVE_SETIMAGEOPTION)
+static VALUE
+arg_is_integer(VALUE arg)
+{
+    int d = NUM2INT(arg);
+    d = d;      // satisfy icc
+    return arg;
+}
+#endif
+
+/*
+    Method:     Info#delay=
+    Purpose:    Set the delay attribute
+    Notes:      Convert from numeric value to string.
+*/
+VALUE
+Info_delay_eq(VALUE self, VALUE string)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    int delay;
+    int not_num;
+    char dstr[20];
+
+    Data_Get_Struct(self, Info, info);
+
+    if (NIL_P(string))
+    {
+        (void) RemoveImageOption(info, "delay");
+    }
+    else
+    {
+        not_num = 0;
+        (void) rb_protect(arg_is_integer, string, &not_num);
+        if (not_num)
+        {
+            rb_raise(rb_eTypeError, "failed to convert %s into Integer", rb_class2name(CLASS_OF(string)));
+        }
+        delay = NUM2INT(string);
+        sprintf(dstr, "%d", delay);
+        (void) SetImageOption(info, "delay", dstr);
+    }
+    return self;
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
 
 DEF_ATTR_READER(Info, density, str)
 
@@ -396,7 +568,7 @@ Info_density_eq(VALUE self, VALUE density_arg)
         return self;
     }
 
-    density = rb_funcall(density_arg, ID_to_s, 0);
+    density = rb_funcall(density_arg, rm_ID_to_s, 0);
     dens = STRING_PTR(density);
     if (!IsGeometry(dens))
     {
@@ -419,10 +591,10 @@ VALUE
 Info_depth_eq(VALUE self, VALUE depth)
 {
     Info *info;
-    int d;
+    unsigned long d;
 
     Data_Get_Struct(self, Info, info);
-    d = NUM2INT(depth);
+    d = NUM2ULONG(depth);
     switch (d)
     {
         case 8:                     // always okay
@@ -442,6 +614,107 @@ Info_depth_eq(VALUE self, VALUE depth)
     return self;
 }
 
+/*
+    Method:     Info#dispose
+    Purpose:    Retrieve a dispose option string and convert it to
+                a DisposeType enumerator
+*/
+#if defined(HAVE_SETIMAGEOPTION)
+static struct
+{
+    char *string;
+    char *enum_name;
+    DisposeType enumerator;
+} Dispose_Option[] = {
+    { "Background", "BackgroundDispose", BackgroundDispose },
+    { "None",       "NoneDispose",       NoneDispose },
+    { "Previous",   "PreviousDispose",   PreviousDispose },
+    { "Undefined",  "UndefinedDispose",  UndefinedDispose },
+    { "0",          "UndefinedDispose",  UndefinedDispose },
+    { "1",          "NoneDispose",       NoneDispose },
+    { "2",          "BackgroundDispose", BackgroundDispose },
+    { "3",          "PreviousDispose",   PreviousDispose },
+    };
+#define N_DISPOSE_OPTIONS (sizeof(Dispose_Option)/sizeof(Dispose_Option[0]))
+#endif
+
+VALUE
+Info_dispose(VALUE self)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    int x;
+    ID dispose_id;
+    const char *dispose;
+
+    Data_Get_Struct(self, Info, info);
+
+    dispose_id = rb_intern("UndefinedDispose");
+
+    // Map the dispose option string to a DisposeType enumerator.
+    dispose=GetImageOption(info, "dispose");
+    if (dispose)
+    {
+        for (x = 0; x < N_DISPOSE_OPTIONS; x++)
+        {
+            if (strcmp(dispose, Dispose_Option[x].string) == 0)
+            {
+                dispose_id = rb_intern(Dispose_Option[x].enum_name);
+                break;
+            }
+        }
+    }
+
+    return rb_const_get(Module_Magick, dispose_id);
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+/*
+    Method:     Info#dispose=
+    Purpose:    Convert a DisposeType enumerator into the equivalent
+                dispose option string
+*/
+VALUE
+Info_dispose_eq(VALUE self, VALUE disp)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    DisposeType dispose;
+    char *option;
+    int x;
+
+    Data_Get_Struct(self, Info, info);
+
+    if (NIL_P(disp))
+    {
+        (void) RemoveImageOption(info, "dispose");
+        return self;
+    }
+
+    VALUE_TO_ENUM(disp, dispose, DisposeType);
+    option = "Undefined";
+
+    for(x = 0; x < N_DISPOSE_OPTIONS; x++)
+    {
+        if (dispose == Dispose_Option[x].enumerator)
+        {
+            option = Dispose_Option[x].string;
+            break;
+        }
+    }
+
+    (void) SetImageOption(info, "dispose", option);
+    return self;
+
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
 DEF_ATTR_ACCESSOR(Info, dither, bool)
 
 #ifdef HAVE_IMAGE_EXTRACT_INFO
@@ -471,7 +744,7 @@ Info_extract_eq(VALUE self, VALUE extract_arg)
         return self;
     }
 
-    extract = rb_funcall(extract_arg, ID_to_s, 0);
+    extract = rb_funcall(extract_arg, rm_ID_to_s, 0);
     extr = STRING_PTR(extract);
     if (!IsGeometry(extr))
     {
@@ -553,7 +826,7 @@ Info_tile_eq(VALUE self, VALUE tile_arg)
         return self;
     }
 
-    tile = rb_funcall(tile_arg, ID_to_s, 0);
+    tile = rb_funcall(tile_arg, rm_ID_to_s, 0);
     til = STRING_PTR(tile);
     if (!IsGeometry(til))
     {
@@ -604,6 +877,53 @@ Info_filename_eq(VALUE self, VALUE filename)
     return self;
 }
 
+
+/*
+    Method:     Info#fill
+    Purpose:    return the fill (a.k.a pen) color as a String
+    Note:       Compare with Image#fill!
+*/
+VALUE
+Info_fill(VALUE self)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    const char *fill;
+
+    Data_Get_Struct(self, Info, info);
+    fill = GetImageOption(info, "fill");
+    return fill ? rb_str_new2(fill) : Qnil;
+#else
+    Info *info;
+
+    Data_Get_Struct(self, Info, info);
+    return PixelPacket_to_Color_Name_Info(info, &info->pen);
+#endif
+}
+
+/*
+    Method:     Info#fill=<aString>
+    Purpose:    set the fill (a.k.a. pen) color
+    Raises:     ArgumentError
+*/
+VALUE
+Info_fill_eq(VALUE self, VALUE color)
+{
+    Info *info;
+
+    Data_Get_Struct(self, Info, info);
+    Color_to_PixelPacket(&info->pen, color);
+
+#if defined(HAVE_SETIMAGEOPTION)
+    // Color_to_PixelPacket will raise an exception if `color' isn't a real color.
+    (void) RemoveImageOption(info, "fill");
+    (void) SetImageOption(info, "fill", STRING_PTR(color));
+#endif
+
+    return self;
+}
+
+
 /*
     Methods:    aString=Info#font
                 Info#font=aString
@@ -646,6 +966,8 @@ VALUE Info_format(VALUE self)
     {
         GetExceptionInfo(&exception);
         magick_info = GetMagickInfo(info->magick, &exception);
+        (void) DestroyExceptionInfo(&exception);
+
         return magick_info ? rb_str_new2(magick_info->name) : Qnil;
     }
 
@@ -670,7 +992,8 @@ Info_format_eq(VALUE self, VALUE magick)
 
     mgk = STRING_PTR(magick);
     m = GetMagickInfo(mgk, &exception);
-    HANDLE_ERROR
+    CHECK_EXCEPTION()
+    (void) DestroyExceptionInfo(&exception);
 
     if (!m)
     {
@@ -697,6 +1020,112 @@ VALUE Info_fuzz_eq(VALUE self, VALUE fuzz)
     return self;
 }
 
+/*
+    Method:     Info#gravity
+    Purpose:    Return the value of the gravity option as a GravityType enumerator
+*/
+
+#if defined(HAVE_SETIMAGEOPTION)
+static struct
+{
+    char *string;
+    char *enum_name;
+    GravityType enumerator;
+} Gravity_Option[] = {
+    { "Undefined",  "UndefinedGravity", UndefinedGravity },
+    { "None",       "UndefinedGravity", UndefinedGravity },
+    { "Center",     "CenterGravity",    CenterGravity },
+    { "East",       "EastGravity",      EastGravity},
+    { "Forget",     "ForgetGravity",    ForgetGravity },
+    { "NorthEast",  "NorthEastGravity", NorthEastGravity },
+    { "North",      "NorthGravity",     NorthGravity },
+    { "NorthWest",  "NorthWestGravity", NorthWestGravity },
+    { "SouthEast",  "SouthEastGravity", SouthEastGravity },
+    { "South",      "SouthGravity",     SouthGravity },
+    { "SouthWest",  "SouthWestGravity", SouthWestGravity },
+    { "West",       "WestGravity",      WestGravity },
+    { "Static",     "StaticGravity",    StaticGravity }
+    };
+#define N_GRAVITY_OPTIONS (sizeof(Gravity_Option)/sizeof(Gravity_Option[0]))
+#endif
+
+VALUE Info_gravity(VALUE self)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    const char *gravity;
+    int x;
+    ID gravity_id;
+
+    Data_Get_Struct(self, Info, info);
+
+    gravity_id = rb_intern("UndefinedGravity");
+
+    // Map the gravity option string to a GravityType enumerator.
+    gravity=GetImageOption(info, "gravity");
+    if (gravity)
+    {
+        for (x = 0; x < N_GRAVITY_OPTIONS; x++)
+        {
+            if (strcmp(gravity, Gravity_Option[x].string) == 0)
+            {
+                gravity_id = rb_intern(Gravity_Option[x].enum_name);
+                break;
+            }
+        }
+    }
+
+    return rb_const_get(Module_Magick, gravity_id);
+
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+/*
+    Method:     Info#gravity=
+    Purpose:    Convert a GravityType enum to a gravity option name and
+                store in the Info structure
+*/
+VALUE
+Info_gravity_eq(VALUE self, VALUE grav)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    GravityType gravity;
+    char *option;
+    int x;
+
+    Data_Get_Struct(self, Info, info);
+
+    if (NIL_P(grav))
+    {
+        (void) RemoveImageOption(info, "gravity");
+        return self;
+    }
+
+    VALUE_TO_ENUM(grav, gravity, GravityType);
+    option = "Undefined";
+
+    for(x = 0; x < N_GRAVITY_OPTIONS; x++)
+    {
+        if (gravity == Gravity_Option[x].enumerator)
+        {
+            option = Gravity_Option[x].string;
+            break;
+        }
+    }
+
+    (void) SetImageOption(info, "gravity", option);
+    return self;
+
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
 
 DEF_ATTR_ACCESSOR(Info, group, long)
 
@@ -758,6 +1187,8 @@ Info_interlace_eq(VALUE self, VALUE inter)
     return self;
 }
 
+OPTION_ATTR_ACCESSOR(label, Label)
+
 /*
     Method:     Info#matte_color
     Purpose:    return the name of the matte color as a String
@@ -787,6 +1218,39 @@ Info_matte_color_eq(VALUE self, VALUE matte_arg)
     return self;
 }
 
+/*
+    Method:     Info#monitor=
+    Purpose:    Establish a progress monitor
+    Notes:      See Image_monitor_eq
+*/
+VALUE
+Info_monitor_eq(VALUE self, VALUE monitor)
+{
+#if defined(HAVE_SETIMAGEPROGRESSMONITOR)
+    Info *info;
+
+    Data_Get_Struct(self, Info, info);
+
+    if (NIL_P(monitor))
+    {
+        info->progress_monitor = NULL;
+    }
+    else
+    {
+        (void) SetImageInfoProgressMonitor(info, rm_progress_monitor, (void *)monitor);
+    }
+
+
+    return self;
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
+
+
 DEF_ATTR_ACCESSOR(Info, monochrome, bool)
 
 #ifdef HAVE_IMAGEINFO_NUMBER_SCENES
@@ -811,7 +1275,123 @@ Info_number_scenes_eq(VALUE self, VALUE nscenes)
 }
 #endif
 
-DEF_ATTR_READER(Info, page, str)
+/*
+    Method:     Info#orientation
+    Purpose:    Return the orientation attribute as an OrientationType enum value.
+*/
+VALUE
+Info_orientation(VALUE self)
+{
+#if defined(HAVE_IMAGEINFO_ORIENTATION)
+    Info *info;
+
+    Data_Get_Struct(self, Info, info);
+    return OrientationType_new(info->orientation);
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
+/*
+    Method:     Info#Orientation=
+    Purpose:    Set the Orientation type
+    Raises:     ArgumentError
+*/
+VALUE
+Info_orientation_eq(VALUE self, VALUE inter)
+{
+#if defined(HAVE_IMAGEINFO_ORIENTATION)
+    Info *info;
+
+    Data_Get_Struct(self, Info, info);
+    VALUE_TO_ENUM(inter, info->orientation, OrientationType);
+    return self;
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
+/*
+    Method:     Info#origin
+    Purpose:    Return origin geometry
+*/
+VALUE
+Info_origin(VALUE self)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    const char *origin;
+
+    Data_Get_Struct(self, Info, info);
+
+    origin = GetImageOption(info, "origin");
+    return origin ? rb_str_new2(origin) : Qnil;
+
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
+/*
+    Method:     Info#origin=+-x+-y
+    Purpose:    Set origin geometry. Argument may be a Geometry object as well
+                as a geometry string.
+*/
+VALUE
+Info_origin_eq(VALUE self, VALUE origin_arg)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    volatile VALUE origin_str;
+    char *origin;
+
+    Data_Get_Struct(self, Info, info);
+
+    if (NIL_P(origin_arg))
+    {
+        (void) RemoveImageOption(info, "origin");
+        return self;
+    }
+
+    origin_str = rb_funcall(origin_arg, rm_ID_to_s, 0);
+    origin = GetPageGeometry(STRING_PTR(origin_str));
+
+    if (IsGeometry(origin) == MagickFalse)
+    {
+        rb_raise(rb_eArgError, "invalid origin geometry: %s", origin);
+    }
+
+    (void) SetImageOption(info, "origin", origin);
+    return self;
+
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
+VALUE
+Info_page(VALUE self)
+{
+    Info *info;
+    const char *page;
+
+    Data_Get_Struct(self, Info, info);
+#if defined(HAVE_SETIMAGEOPTION)
+    page = GetImageOption(info, "page");
+#else
+    page = (const char *)info->page;
+#endif
+    return page ? rb_str_new2(page) : Qnil;
+
+}
 
 /*
     Method:     Info#page=<aString> or <aGeometry>
@@ -831,8 +1411,8 @@ Info_page_eq(VALUE self, VALUE page_arg)
         info->page = NULL;
         return self;
     }
-    geom_str = rb_funcall(page_arg, ID_to_s, 0);
-    geometry=PostscriptGeometry(STRING_PTR(geom_str));
+    geom_str = rb_funcall(page_arg, rm_ID_to_s, 0);
+    geometry=GetPageGeometry(STRING_PTR(geom_str));
     if (*geometry == '\0')
     {
         magick_free(info->page);
@@ -840,11 +1420,63 @@ Info_page_eq(VALUE self, VALUE page_arg)
         return self;
     }
     magick_clone_string(&info->page, geometry);
+#if defined(HAVE_SETIMAGEOPTION)
+    (void) SetImageOption(info, "page", STRING_PTR(geom_str));
+#endif
 
     return self;
 }
 
-DEF_ATTR_ACCESSOR(Info, quality, long)
+DEF_ATTR_ACCESSOR(Info, pointsize, dbl)
+DEF_ATTR_ACCESSOR(Info, quality, ulong)
+
+/*
+    Method:     Info#sampling_factor, #sampling_factor=
+    Purpose:    get/set sampling factors used by JPEG or MPEG-2 encoder
+                and YUV decoder/encoder
+*/
+VALUE
+Info_sampling_factor(VALUE self)
+{
+    Info *info;
+
+    Data_Get_Struct(self, Info, info);
+    if (info->sampling_factor)
+    {
+        return rb_str_new2(info->sampling_factor);
+    }
+    else
+    {
+        return Qnil;
+    }
+}
+
+VALUE
+Info_sampling_factor_eq(VALUE self, VALUE sampling_factor)
+{
+    Info *info;
+    char *sampling_factor_p = NULL;
+    long sampling_factor_len = 0;
+
+    Data_Get_Struct(self, Info, info);
+
+    if (!NIL_P(sampling_factor))
+    {
+        sampling_factor_p = STRING_PTR_LEN(sampling_factor, sampling_factor_len);
+    }
+
+    if (info->sampling_factor)
+    {
+        magick_free(info->sampling_factor);
+        info->sampling_factor = NULL;
+    }
+    if (sampling_factor_len > 0)
+    {
+        magick_clone_string(&info->sampling_factor, sampling_factor_p);
+    }
+
+    return self;
+}
 
 #ifdef HAVE_IMAGEINFO_NUMBER_SCENES
 
@@ -977,7 +1609,7 @@ Info_size_eq(VALUE self, VALUE size_arg)
         return self;
     }
 
-    size = rb_funcall(size_arg, ID_to_s, 0);
+    size = rb_funcall(size_arg, rm_ID_to_s, 0);
     sz = STRING_PTR(size);
     if (!IsGeometry(sz))
     {
@@ -990,6 +1622,43 @@ Info_size_eq(VALUE self, VALUE size_arg)
 }
 
 
+/*
+    Method:     Image::Info#texture=texture_image
+    Purpose:    Set name of texture to tile onto the image background
+*/
+VALUE
+Info_texture_eq(VALUE self, VALUE texture)
+{
+    Info *info;
+    Image *image;
+    char name[MaxTextExtent];
+
+    Data_Get_Struct(self, Info, info);
+
+    // Delete any existing texture file
+    if (info->texture)
+    {
+        rm_delete_temp_image(info->texture);
+        magick_free(info->texture);
+        info->texture = NULL;
+    }
+
+    // If argument is nil we're done
+    if (texture == Qnil)
+    {
+        return self;
+    }
+
+    // Create a temp copy of the texture and store its name in the texture field
+    Data_Get_Struct(texture, Image, image);
+    rm_write_temp_image(image, name);
+
+    magick_clone_string(&info->texture, name);
+
+    return self;
+}
+
+
 /*
     Method:     Info#undefine
     Purpose:    Undefine image option
@@ -1106,6 +1775,27 @@ Info_view_eq(VALUE self, VALUE view_arg)
     return self;
 }
 
+
+/*
+    Static:     destroy_Info
+    Purpose:    if there is a texture image, delete it before destroying
+                the ImageInfo structure
+*/
+static void
+destroy_Info(void *infoptr)
+{
+    Info *info = (Info *)infoptr;
+
+    if (info->texture)
+    {
+        rm_delete_temp_image(info->texture);
+        magick_free(info->texture);
+        info->texture = NULL;
+    }
+
+    (void) DestroyImageInfo(info);
+}
+
 /*
     Method:     Info.new
     Purpose:    Create an Info object by calling CloneInfo
@@ -1122,7 +1812,7 @@ Info_new(VALUE class)
     {
         rb_raise(rb_eNoMemError, "not enough memory to initialize Info object");
     }
-    new_obj = Data_Wrap_Struct(class, NULL, DestroyImageInfo, info);
+    new_obj = Data_Wrap_Struct(class, NULL, destroy_Info, info);
     rb_obj_call_init(new_obj, 0, NULL);
     return new_obj;
 }
@@ -1154,7 +1844,7 @@ Info_alloc(VALUE class)
     {
         rb_raise(rb_eNoMemError, "not enough memory to initialize Info object");
     }
-    info_obj = Data_Wrap_Struct(class, NULL, DestroyImageInfo, info);
+    info_obj = Data_Wrap_Struct(class, NULL, destroy_Info, info);
     return info_obj;
 }
 /*
@@ -1182,7 +1872,64 @@ Info_initialize(VALUE self)
     if (rb_block_given_p())
     {
         // Run the block in self's context
-        rb_obj_instance_eval(0, NULL, self);
+        (void) rb_obj_instance_eval(0, NULL, self);
     }
     return self;
 }
+
+
+/*
+    Method:     Info#get_option
+    Purpose:    Return the value of the specified option
+*/
+static VALUE
+get_option(VALUE self, char *key)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    const char *value;
+
+    Data_Get_Struct(self, Info, info);
+
+    value = GetImageOption(info, key);
+    if (value)
+    {
+        return rb_str_new2(value);
+    }
+    return Qnil;
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+/*
+    Method:     Info#set_option
+    Purpose:    Set the specified option to this value.
+                If the value is nil just unset any current value
+*/
+static VALUE
+set_option(VALUE self, char *key, VALUE string)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    char *value;
+
+    Data_Get_Struct(self, Info, info);
+
+    if (NIL_P(string))
+    {
+        (void) RemoveImageOption(info, key);
+    }
+    else
+    {
+        value = STRING_PTR(string);
+        (void) SetImageOption(info, key, value);
+    }
+    return self;
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
