@@ -1,6 +1,6 @@
-/* $Id: rmmain.c,v 1.120 2006/06/02 23:26:53 rmagick Exp $ */
+/* $Id: rmmain.c,v 1.159.2.2 2007/06/09 16:45:25 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2006 by Timothy P. Hunter
+|                Copyright (C) 2007 by Timothy P. Hunter
 | Name:     rmmain.c
 | Author:   Tim Hunter
 | Purpose:  Contains all module, class, method declarations.
@@ -44,7 +44,7 @@ Magick_colors(VALUE class)
 
     color_ary = GetColorInfoArray(&exception);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
 
     if (rb_block_given_p())
@@ -89,7 +89,7 @@ Magick_colors(VALUE class)
 
     color_info_list = GetColorInfoList("*", &number_colors, &exception);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
 #endif
 
@@ -98,7 +98,7 @@ Magick_colors(VALUE class)
     {
         for(x = 0; x < number_colors; x++)
         {
-            rb_yield(Color_from_ColorInfo(color_info_list[x]));
+            (void) rb_yield(Color_from_ColorInfo(color_info_list[x]));
         }
         magick_free(color_info_list);
         return class;
@@ -108,7 +108,7 @@ Magick_colors(VALUE class)
         ary = rb_ary_new2((long) number_colors);
         for(x = 0; x < number_colors; x++)
         {
-            rb_ary_push(ary, Color_from_ColorInfo(color_info_list[x]));
+            (void) rb_ary_push(ary, Color_from_ColorInfo(color_info_list[x]));
         }
 
         magick_free(color_info_list);
@@ -125,7 +125,7 @@ Magick_colors(VALUE class)
     GetExceptionInfo(&exception);
 
     color_list = GetColorInfo("*", &exception);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     // The order of the colors list can change in mid-iteration,
     // so the only way we can guarantee a single pass thru the list
@@ -179,7 +179,7 @@ Magick_fonts(VALUE class)
     GetExceptionInfo(&exception);
     type_info = GetTypeInfoList("*", &number_types, &exception);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
 #endif
 
@@ -187,17 +187,17 @@ Magick_fonts(VALUE class)
     {
         for(x = 0; x < number_types; x++)
         {
-            rb_yield(Font_from_TypeInfo((TypeInfo *)type_info[x]));
+            (void) rb_yield(Font_from_TypeInfo((TypeInfo *)type_info[x]));
         }
         magick_free(type_info);
         return class;
     }
     else
     {
-        ary = rb_ary_new2(number_types);
+        ary = rb_ary_new2((long)number_types);
         for(x = 0; x < number_types; x++)
         {
-            rb_ary_push(ary, Font_from_TypeInfo((TypeInfo *)type_info[x]));
+            (void) rb_ary_push(ary, Font_from_TypeInfo((TypeInfo *)type_info[x]));
         }
         magick_free(type_info);
         return ary;
@@ -214,7 +214,7 @@ Magick_fonts(VALUE class)
 
     type_list = GetTypeInfo("*", &exception);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     // If block, iterate over fonts
     if (rb_block_given_p())
@@ -288,7 +288,7 @@ Magick_init_formats(VALUE class)
     GetExceptionInfo(&exception);
     magick_info = (MagickInfo *)GetMagickInfoArray(&exception);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     for(m = magick_info; m != NULL; m = m->next)
     {
@@ -318,15 +318,15 @@ Magick_init_formats(VALUE class)
     GetExceptionInfo(&exception);
     magick_info = GetMagickInfoList("*", &number_formats, &exception);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
 #endif
 
     for(x = 0; x < number_formats; x++)
     {
-        rb_hash_aset(formats
-            , rb_str_new2(magick_info[x]->name)
-            , MagickInfo_to_format((MagickInfo *)magick_info[x]));
+        (void) rb_hash_aset(formats
+                          , rb_str_new2(magick_info[x]->name)
+                          , MagickInfo_to_format((MagickInfo *)magick_info[x]));
     }
     return formats;
 
@@ -343,7 +343,7 @@ Magick_init_formats(VALUE class)
     GetExceptionInfo(&exception);
     m = (MagickInfo *)GetMagickInfo("*", &exception);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     for ( ; m != NULL; m = m->next)
     {
@@ -380,7 +380,7 @@ Magick_limit_resource(int argc, VALUE *argv, VALUE class)
             return class;
 
         case T_SYMBOL:
-            id = SYM2ID(resource);
+            id = (ID)SYM2ID(resource);
             if (id == rb_intern("memory"))
             {
                 res = MemoryResource;
@@ -436,7 +436,7 @@ Magick_limit_resource(int argc, VALUE *argv, VALUE class)
 
     if (argc > 1)
     {
-        SetMagickResourceLimit(res, NUM2ULONG(limit));
+        (void) SetMagickResourceLimit(res, NUM2ULONG(limit));
     }
 
     return ULONG2NUM(cur_limit);
@@ -463,7 +463,7 @@ monitor_handler(
 
     exception = exception;  // defeat "never referenced" message from icc
 
-    if (rb_cvar_defined(Module_Magick, Magick_Monitor))
+    if (rb_cvar_defined(Module_Magick, Magick_Monitor) == Qtrue)
     {
         args[0] = rb_str_new2(text);
         // Convert these possibly-64-bit types to 32-bit types that
@@ -472,7 +472,7 @@ monitor_handler(
         args[2] = UINT2NUM((unsigned long) span);
 
         monitor = rb_cvar_get(Module_Magick, Magick_Monitor);
-        (void) rb_funcall2((VALUE)monitor, ID_call, 3, (VALUE *)args);
+        (void) rb_funcall2((VALUE)monitor, rm_ID_call, 3, (VALUE *)args);
     }
 
     return True;
@@ -532,8 +532,8 @@ static VALUE
 Magick_set_cache_threshold(VALUE class, VALUE threshold)
 {
     unsigned long thrshld = NUM2ULONG(threshold);
-    SetMagickResourceLimit(MemoryResource,thrshld);
-    SetMagickResourceLimit(MapResource,2*thrshld);
+    (void) SetMagickResourceLimit(MemoryResource,thrshld);
+    (void) SetMagickResourceLimit(MapResource,2*thrshld);
     return class;
 }
 
@@ -604,29 +604,30 @@ Init_RMagick(void)
     /* Create IDs for frequently used methods, etc.                          */
     /*-----------------------------------------------------------------------*/
 
-    ID__dummy_img_      = rb_intern("_dummy_img_");
-    ID_call             = rb_intern("call");
-    ID_changed          = rb_intern("changed");
-    ID_cur_image        = rb_intern("cur_image");
-    ID_dup              = rb_intern("dup");
-    ID_enumerators      = rb_intern("enumerators");
-    ID_fill             = rb_intern("fill");
-    ID_flag             = rb_intern("flag");
-    ID_from_s           = rb_intern("from_s");
-    ID_Geometry         = rb_intern("Geometry");
-    ID_GeometryValue    = rb_intern("GeometryValue");
-    ID_height           = rb_intern("height");
-    ID_initialize_copy  = rb_intern("initialize_copy");
-    ID_length           = rb_intern("length");
-    ID_notify_observers = rb_intern("notify_observers");
-    ID_new              = rb_intern("new");
-    ID_push             = rb_intern("push");
-    ID_spaceship        = rb_intern("<=>");
-    ID_to_s             = rb_intern("to_s");
-    ID_values           = rb_intern("values");
-    ID_width            = rb_intern("width");
-    ID_x                = rb_intern("x");
-    ID_y                = rb_intern("y");
+    rm_ID__dummy_img_      = rb_intern("_dummy_img_");
+    rm_ID_call             = rb_intern("call");
+    rm_ID_changed          = rb_intern("changed");
+    rm_ID_cur_image        = rb_intern("cur_image");
+    rm_ID_dup              = rb_intern("dup");
+    rm_ID_enumerators      = rb_intern("enumerators");
+    rm_ID_fill             = rb_intern("fill");
+    rm_ID_flag             = rb_intern("flag");
+    rm_ID_from_s           = rb_intern("from_s");
+    rm_ID_Geometry         = rb_intern("Geometry");
+    rm_ID_GeometryValue    = rb_intern("GeometryValue");
+    rm_ID_height           = rb_intern("height");
+    rm_ID_initialize_copy  = rb_intern("initialize_copy");
+    rm_ID_length           = rb_intern("length");
+    rm_ID_notify_observers = rb_intern("notify_observers");
+    rm_ID_new              = rb_intern("new");
+    rm_ID_push             = rb_intern("push");
+    rm_ID_spaceship        = rb_intern("<=>");
+    rm_ID__tmpnam_         = rb_intern("@@_tmpnam_");
+    rm_ID_to_s             = rb_intern("to_s");
+    rm_ID_values           = rb_intern("values");
+    rm_ID_width            = rb_intern("width");
+    rm_ID_x                = rb_intern("x");
+    rm_ID_y                = rb_intern("y");
 
     /*-----------------------------------------------------------------------*/
     /* Module Magick methods                                                 */
@@ -670,11 +671,12 @@ Init_RMagick(void)
     DCL_ATTR_READER(Image, base_columns)
     DCL_ATTR_READER(Image, base_filename)
     DCL_ATTR_READER(Image, base_rows)
+    DCL_ATTR_ACCESSOR(Image, bias)
+    DCL_ATTR_ACCESSOR(Image, black_point_compensation)
     DCL_ATTR_ACCESSOR(Image, blur)
     DCL_ATTR_ACCESSOR(Image, border_color)
     DCL_ATTR_READER(Image, bounding_box)
     DCL_ATTR_ACCESSOR(Image, chromaticity)
-    DCL_ATTR_WRITER(Image, clip_mask)
     DCL_ATTR_ACCESSOR(Image, color_profile)
     DCL_ATTR_READER(Image, colors)
     DCL_ATTR_ACCESSOR(Image, colorspace)
@@ -699,6 +701,7 @@ Init_RMagick(void)
     DCL_ATTR_ACCESSOR(Image, interlace)
     DCL_ATTR_ACCESSOR(Image, iptc_profile)
     DCL_ATTR_ACCESSOR(Image, iterations)        // do not document! Only used by Image#iterations=
+    DCL_ATTR_ACCESSOR(Image, mask)
     DCL_ATTR_ACCESSOR(Image, matte)
     DCL_ATTR_ACCESSOR(Image, matte_color)
     DCL_ATTR_READER(Image, mean_error_per_pixel)
@@ -710,8 +713,9 @@ Init_RMagick(void)
     DCL_ATTR_READER(Image, number_colors)
     DCL_ATTR_ACCESSOR(Image, offset)
     DCL_ATTR_WRITER(Image, opacity)
-    DCL_ATTR_READER(Image, orientation)
+    DCL_ATTR_ACCESSOR(Image, orientation)
     DCL_ATTR_ACCESSOR(Image, page)
+    DCL_ATTR_ACCESSOR(Image, pixel_interpolation_method)
 #if defined(HAVE_IMAGE_QUALITY)
     DCL_ATTR_READER(Image, quality)
 #endif
@@ -724,20 +728,30 @@ Init_RMagick(void)
     DCL_ATTR_ACCESSOR(Image, ticks_per_second)
     DCL_ATTR_ACCESSOR(Image, tile_info)
     DCL_ATTR_READER(Image, total_colors)
+    DCL_ATTR_ACCESSOR(Image, transparent_color)
     DCL_ATTR_ACCESSOR(Image, units)
     DCL_ATTR_ACCESSOR(Image, virtual_pixel_method)
     DCL_ATTR_ACCESSOR(Image, x_resolution)
     DCL_ATTR_ACCESSOR(Image, y_resolution)
 
+    rb_define_method(Class_Image, "adaptive_blur", Image_adaptive_blur, -1);
+    rb_define_method(Class_Image, "adaptive_blur_channel", Image_adaptive_blur_channel, -1);
+    rb_define_method(Class_Image, "adaptive_resize", Image_adaptive_resize, -1);
+    rb_define_method(Class_Image, "adaptive_sharpen", Image_adaptive_sharpen, -1);
+    rb_define_method(Class_Image, "adaptive_sharpen_channel", Image_adaptive_sharpen_channel, -1);
     rb_define_method(Class_Image, "adaptive_threshold", Image_adaptive_threshold, -1);
     rb_define_method(Class_Image, "add_noise", Image_add_noise, 1);
     rb_define_method(Class_Image, "add_noise_channel", Image_add_noise_channel, -1);
+    rb_define_method(Class_Image, "add_profile", Image_add_profile, 1);
     rb_define_method(Class_Image, "affine_transform", Image_affine_transform, 1);
     rb_define_method(Class_Image, "[]", Image_aref, 1);
     rb_define_method(Class_Image, "[]=", Image_aset, 2);
+    rb_define_method(Class_Image, "auto_orient", Image_auto_orient, 0);
+    rb_define_method(Class_Image, "auto_orient!", Image_auto_orient_bang, 0);
     rb_define_method(Class_Image, "properties", Image_properties, 0);
     rb_define_method(Class_Image, "bilevel_channel", Image_bilevel_channel, -1);
     rb_define_method(Class_Image, "black_threshold", Image_black_threshold, -1);
+    rb_define_method(Class_Image, "blend", Image_blend, -1);
     rb_define_method(Class_Image, "blur_image", Image_blur_image, -1);
     rb_define_method(Class_Image, "blur_channel", Image_blur_channel, -1);
     rb_define_method(Class_Image, "border", Image_border, 3);
@@ -763,6 +777,8 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "composite", Image_composite, -1);
     rb_define_method(Class_Image, "composite!", Image_composite_bang, -1);
     rb_define_method(Class_Image, "composite_affine", Image_composite_affine, 2);
+    rb_define_method(Class_Image, "composite_channel", Image_composite_channel, -1);
+    rb_define_method(Class_Image, "composite_channel!", Image_composite_channel_bang, -1);
     rb_define_method(Class_Image, "compress_colormap!", Image_compress_colormap_bang, 0);
     rb_define_method(Class_Image, "contrast", Image_contrast, -1);
     rb_define_method(Class_Image, "contrast_stretch_channel", Image_contrast_stretch_channel, -1);
@@ -772,10 +788,13 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "crop", Image_crop, -1);
     rb_define_method(Class_Image, "crop!", Image_crop_bang, -1);
     rb_define_method(Class_Image, "cycle_colormap", Image_cycle_colormap, 1);
+    rb_define_method(Class_Image, "delete_profile", Image_delete_profile, 1);
     rb_define_method(Class_Image, "despeckle", Image_despeckle, 0);
     rb_define_method(Class_Image, "difference", Image_difference, 1);
     rb_define_method(Class_Image, "dispatch", Image_dispatch, -1);
+    rb_define_method(Class_Image, "displace", Image_displace, -1);
     rb_define_method(Class_Image, "display", Image_display, 0);
+    rb_define_method(Class_Image, "dissolve", Image_dissolve, -1);
     rb_define_method(Class_Image, "distortion_channel", Image_distortion_channel, -1);
     rb_define_method(Class_Image, "_dump", Image__dump, 1);
     rb_define_method(Class_Image, "dup", Image_dup, 0);
@@ -787,6 +806,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "erase!", Image_erase_bang, 0);
     rb_define_method(Class_Image, "export_pixels", Image_export_pixels, -1);
     rb_define_method(Class_Image, "export_pixels_to_str", Image_export_pixels_to_str, -1);
+    rb_define_method(Class_Image, "find_similar_region", Image_find_similar_region, -1);
     rb_define_method(Class_Image, "flip", Image_flip, 0);
     rb_define_method(Class_Image, "flip!", Image_flip_bang, 0);
     rb_define_method(Class_Image, "flop", Image_flop, 0);
@@ -806,6 +826,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "inspect", Image_inspect, 0);
     rb_define_method(Class_Image, "level2", Image_level2, -1);
     rb_define_method(Class_Image, "level_channel", Image_level_channel, -1);
+    rb_define_method(Class_Image, "linear_stretch", Image_linear_stretch, -1);
     rb_define_method(Class_Image, "magnify", Image_magnify, 0);
     rb_define_method(Class_Image, "magnify!", Image_magnify_bang, 0);
     rb_define_method(Class_Image, "map", Image_map, -1);
@@ -815,7 +836,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "minify!", Image_minify_bang, 0);
     rb_define_method(Class_Image, "modulate", Image_modulate, -1);
     rb_define_method(Class_Image, "monochrome?", Image_monochrome_q, 0);
-    rb_define_method(Class_Image, "motion_blur", Image_motion_blur, 3);
+    rb_define_method(Class_Image, "motion_blur", Image_motion_blur, -1);
     rb_define_method(Class_Image, "negate", Image_negate, -1);
     rb_define_method(Class_Image, "negate_channel", Image_negate_channel, -1);
     rb_define_method(Class_Image, "normalize", Image_normalize, 0);
@@ -826,6 +847,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "ordered_dither", Image_ordered_dither, -1);
     rb_define_method(Class_Image, "palette?", Image_palette_q, 0);
     rb_define_method(Class_Image, "pixel_color", Image_pixel_color, -1);
+    rb_define_method(Class_Image, "polaroid", Image_polaroid, -1);
     rb_define_method(Class_Image, "posterize", Image_posterize, -1);
 //  rb_define_method(Class_Image, "plasma", Image_plasma, 6);
     rb_define_method(Class_Image, "preview", Image_preview, 1);
@@ -833,15 +855,17 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "quantize", Image_quantize, -1);
     rb_define_method(Class_Image, "quantum_operator", Image_quantum_operator, -1);
     rb_define_method(Class_Image, "radial_blur", Image_radial_blur, 1);
+    rb_define_method(Class_Image, "radial_blur_channel", Image_radial_blur_channel, -1);
     rb_define_method(Class_Image, "raise", Image_raise, -1);
     rb_define_method(Class_Image, "random_channel_threshold", Image_random_channel_threshold, 2);
     rb_define_method(Class_Image, "random_threshold_channel", Image_random_threshold_channel, -1);
+    rb_define_method(Class_Image, "recolor", Image_recolor, 1);
     rb_define_method(Class_Image, "reduce_noise", Image_reduce_noise, 1);
     rb_define_method(Class_Image, "resize", Image_resize, -1);
     rb_define_method(Class_Image, "resize!", Image_resize_bang, -1);
     rb_define_method(Class_Image, "roll", Image_roll, 2);
-    rb_define_method(Class_Image, "rotate", Image_rotate, 1);
-    rb_define_method(Class_Image, "rotate!", Image_rotate_bang, 1);
+    rb_define_method(Class_Image, "rotate", Image_rotate, -1);
+    rb_define_method(Class_Image, "rotate!", Image_rotate_bang, -1);
     rb_define_method(Class_Image, "sample", Image_sample, -1);
     rb_define_method(Class_Image, "sample!", Image_sample_bang, -1);
     rb_define_method(Class_Image, "scale", Image_scale, -1);
@@ -858,6 +882,7 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "shear", Image_shear, 2);
     rb_define_method(Class_Image, "sigmoidal_contrast_channel", Image_sigmoidal_contrast_channel, -1);
     rb_define_method(Class_Image, "signature", Image_signature, 0);
+    rb_define_method(Class_Image, "sketch", Image_sketch, -1);
     rb_define_method(Class_Image, "solarize", Image_solarize, -1);
     rb_define_method(Class_Image, "<=>", Image_spaceship, 1);
     rb_define_method(Class_Image, "splice", Image_splice, -1);
@@ -876,12 +901,19 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "to_color", Image_to_color, 1);
     rb_define_method(Class_Image, "to_blob", Image_to_blob, 0);
     rb_define_method(Class_Image, "transparent", Image_transparent, -1);
+    rb_define_method(Class_Image, "transpose", Image_transpose, 0);
+    rb_define_method(Class_Image, "transpose!", Image_transpose_bang, 0);
+    rb_define_method(Class_Image, "transverse", Image_transverse, 0);
+    rb_define_method(Class_Image, "transverse!", Image_transverse_bang, 0);
     rb_define_method(Class_Image, "trim", Image_trim, 0);
     rb_define_method(Class_Image, "trim!", Image_trim_bang, 0);
+    rb_define_method(Class_Image, "unique_colors", Image_unique_colors, 0);
     rb_define_method(Class_Image, "unsharp_mask", Image_unsharp_mask, -1);
     rb_define_method(Class_Image, "unsharp_mask_channel", Image_unsharp_mask_channel, -1);
     rb_define_method(Class_Image, "vignette", Image_vignette, -1);
+    rb_define_method(Class_Image, "watermark", Image_watermark, -1);
     rb_define_method(Class_Image, "wave", Image_wave, -1);
+    rb_define_method(Class_Image, "wet_floor", Image_wet_floor, -1);
     rb_define_method(Class_Image, "white_threshold", Image_white_threshold, -1);
     rb_define_method(Class_Image, "write", Image_write, 1);
 
@@ -904,7 +936,8 @@ Init_RMagick(void)
     rb_define_method(Class_ImageList, "deconstruct", ImageList_deconstruct, 0);
     rb_define_method(Class_ImageList, "display", ImageList_display, 0);
     rb_define_method(Class_ImageList, "flatten_images", ImageList_flatten_images, 0);
-    rb_define_method(Class_ImageList, "map", ImageList_map, 2);
+    rb_define_method(Class_ImageList, "fx", ImageList_fx, -1);
+    rb_define_method(Class_ImageList, "map", ImageList_map, -1);
     rb_define_method(Class_ImageList, "montage", ImageList_montage, 0);
     rb_define_method(Class_ImageList, "morph", ImageList_morph, 1);
     rb_define_method(Class_ImageList, "mosaic", ImageList_mosaic, 0);
@@ -926,7 +959,6 @@ Init_RMagick(void)
 
     DCL_ATTR_WRITER(Draw, affine)
     DCL_ATTR_WRITER(Draw, align)
-    DCL_ATTR_WRITER(Draw, border_color)
     DCL_ATTR_WRITER(Draw, decorate)
     DCL_ATTR_WRITER(Draw, density)
     DCL_ATTR_WRITER(Draw, encoding)
@@ -942,6 +974,7 @@ Init_RMagick(void)
     DCL_ATTR_WRITER(Draw, stroke)
     DCL_ATTR_WRITER(Draw, stroke_width)
     DCL_ATTR_WRITER(Draw, text_antialias)
+    DCL_ATTR_WRITER(Draw, tile)
     DCL_ATTR_WRITER(Draw, undercolor)
 
     rb_define_method(Class_Draw, "annotate", Draw_annotate, 6);
@@ -956,6 +989,42 @@ Init_RMagick(void)
     rb_define_method(Class_Draw, "inspect", Draw_inspect, 0);
     rb_define_method(Class_Draw, "primitive", Draw_primitive, 1);
 
+    /*-----------------------------------------------------------------------*/
+    /* Class Magick::DrawOptions is identical to Magick::Draw but with       */
+    /* only the attribute writer methods. This is the object that is passed  */
+    /* to the block associated with the Draw.new method call.                */
+    /*-----------------------------------------------------------------------*/
+
+    Class_DrawOptions = rb_define_class_under(Class_Image, "DrawOptions", rb_cObject);
+
+#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
+    rb_define_alloc_func(Class_DrawOptions, DrawOptions_alloc);
+#else
+    rb_define_singleton_method(Class_DrawOptions, "new", DrawOptions_new, 0);
+#endif
+
+    rb_define_method(Class_DrawOptions, "initialize", DrawOptions_initialize, 0);
+
+    SHARE_ATTR_WRITER(DrawOptions, Draw, affine)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, align)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, decorate)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, density)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, encoding)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, fill)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, font)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, font_family)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, font_stretch)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, font_style)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, font_weight)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, gravity)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, pointsize)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, rotation)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, stroke)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, stroke_width)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, text_antialias)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, tile)
+    SHARE_ATTR_WRITER(DrawOptions, Draw, undercolor)
+
     /*-----------------------------------------------------------------------*/
     /* Class Magick::Pixel                                                   */
     /*-----------------------------------------------------------------------*/
@@ -963,7 +1032,7 @@ Init_RMagick(void)
     Class_Pixel = rb_define_class_under(Module_Magick, "Pixel", rb_cObject);
 
     // include Observable in Pixel for Image::View class
-    rb_require("observer");
+    (void) rb_require("observer");
     observable = rb_const_get(rb_cObject, rb_intern("Observable"));
     rb_include_module(Class_Pixel, observable);
 
@@ -995,11 +1064,13 @@ Init_RMagick(void)
     // Define the instance methods
     rb_define_method(Class_Pixel, "<=>", Pixel_spaceship, 1);
     rb_define_method(Class_Pixel, "===", Pixel_case_eq, 1);
+    rb_define_method(Class_Pixel, "eql?", Pixel_eql_q, 1);
     rb_define_method(Class_Pixel, "initialize", Pixel_initialize, -1);
     rb_define_method(Class_Pixel, "initialize_copy", Pixel_init_copy, 1);
     rb_define_method(Class_Pixel, "clone", Pixel_clone, 0);
     rb_define_method(Class_Pixel, "dup", Pixel_dup, 0);
     rb_define_method(Class_Pixel, "fcmp", Pixel_fcmp, -1);
+    rb_define_method(Class_Pixel, "hash", Pixel_hash, 0);
     rb_define_method(Class_Pixel, "intensity", Pixel_intensity, 0);
     rb_define_method(Class_Pixel, "to_color", Pixel_to_color, -1);
     rb_define_method(Class_Pixel, "to_HSL", Pixel_to_HSL, 0);
@@ -1052,6 +1123,7 @@ Init_RMagick(void)
 #endif
 
     rb_define_method(Class_Info, "initialize", Info_initialize, 0);
+    rb_define_method(Class_Info, "channel", Info_channel, -1);
     rb_define_method(Class_Info, "freeze", rm_no_freeze, 0);
     rb_define_method(Class_Info, "define", Info_define, -1);
     rb_define_method(Class_Info, "[]=", Info_aset, 3);
@@ -1085,6 +1157,7 @@ Init_RMagick(void)
     DCL_ATTR_ACCESSOR(Info, monochrome)
     DCL_ATTR_ACCESSOR(Info, number_scenes)  // new in 5.5.6, replaces subrange
     DCL_ATTR_ACCESSOR(Info, orientation)    // new in 6.0.0
+    DCL_ATTR_ACCESSOR(Info, origin)         // new in 6.3.1
     DCL_ATTR_ACCESSOR(Info, page)
     DCL_ATTR_ACCESSOR(Info, pointsize)
     DCL_ATTR_ACCESSOR(Info, quality)
@@ -1094,11 +1167,47 @@ Init_RMagick(void)
     DCL_ATTR_ACCESSOR(Info, size)
     DCL_ATTR_ACCESSOR(Info, subimage)   // deprecated >=5.5.6, replaced by scene
     DCL_ATTR_ACCESSOR(Info, subrange)   // deprecated >=5.5.6, replaced by number_scenes
+    DCL_ATTR_WRITER(Info, texture)
     DCL_ATTR_ACCESSOR(Info, tile)       // deprecated >=5.5.6, replaced by extract and scenes
     DCL_ATTR_ACCESSOR(Info, image_type)
     DCL_ATTR_ACCESSOR(Info, units)
     DCL_ATTR_ACCESSOR(Info, view)
 
+
+    /*-----------------------------------------------------------------------*/
+    /* Class Magick::Image::PolaroidOptions                                  */
+    /*-----------------------------------------------------------------------*/
+
+    Class_PolaroidOptions = rb_define_class_under(Class_Image, "PolaroidOptions", rb_cObject);
+
+#if defined(HAVE_RB_DEFINE_ALLOC_FUNC)
+    rb_define_alloc_func(Class_PolaroidOptions, PolaroidOptions_alloc);
+#else
+    rb_define_singleton_method(Class_PolaroidOptions, "new", PolaroidOptions_new, 0);
+#endif
+
+    rb_define_method(Class_PolaroidOptions, "initialize", PolaroidOptions_initialize, 0);
+
+    DCL_ATTR_WRITER(PolaroidOptions, shadow_color)
+    // The other attribute writer methods are implemented by Draw's functions
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, align)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, decorate)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, density)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, encoding)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, fill)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, font)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, font_family)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, font_stretch)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, font_style)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, font_weight)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, gravity)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, pointsize)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, stroke)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, stroke_width)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, text_antialias)
+    SHARE_ATTR_WRITER(PolaroidOptions, Draw, undercolor)
+
+
     /*-----------------------------------------------------------------------*/
     /* Magick::******Fill classes and methods                                */
     /*-----------------------------------------------------------------------*/
@@ -1192,6 +1301,8 @@ Init_RMagick(void)
         ENUMERATOR(BlueChannel)
         ENUMERATOR(YellowChannel)
         ENUMERATOR(OpacityChannel)
+
+
         ENUMERATOR(BlackChannel)
         ENUMERATOR(MatteChannel)
 #if defined(HAVE_INDEXCHANNEL)
@@ -1203,6 +1314,29 @@ Init_RMagick(void)
 #if defined(HAVE_ALLCHANNELS)
         ENUMERATOR(AllChannels)
 #endif
+
+        // Define alternate names for ChannelType enums for Image::Info#channel=
+        // AlphaChannel == OpacityChannel
+        _enum = rm_enum_new(Class_ChannelType, ID2SYM(rb_intern("AlphaChannel")), INT2FIX(OpacityChannel));
+        rb_define_const(Module_Magick, "AlphaChannel", _enum);
+
+        // DefaultChannels
+        _enum = rm_enum_new(Class_ChannelType, ID2SYM(rb_intern("DefaultChannels")), INT2FIX(0xff & ~OpacityChannel));
+        rb_define_const(Module_Magick, "DefaultChannels", _enum);
+
+        // HueChannel == RedChannel
+        _enum = rm_enum_new(Class_ChannelType, ID2SYM(rb_intern("HueChannel")), INT2FIX(RedChannel));
+        rb_define_const(Module_Magick, "HueChannel", _enum);
+
+        // LuminosityChannel = BlueChannel
+        _enum = rm_enum_new(Class_ChannelType, ID2SYM(rb_intern("LuminosityChannel")), INT2FIX(BlueChannel));
+        rb_define_const(Module_Magick, "LuminosityChannel", _enum);
+
+        // SaturationChannel = GreenChannel
+        _enum = rm_enum_new(Class_ChannelType, ID2SYM(rb_intern("SaturationChannel")), INT2FIX(GreenChannel));
+        rb_define_const(Module_Magick, "SaturationChannel", _enum);
+
+
     END_ENUM
 
     // ClassType constants
@@ -1288,12 +1422,10 @@ Init_RMagick(void)
         ENUMERATOR(CopyGreenCompositeOp)
         ENUMERATOR(CopyOpacityCompositeOp)
         ENUMERATOR(CopyRedCompositeOp)
-#if defined(HAVE_COPYCYANCOMPOSITEOP)   // CYMK added 5.5.7
         ENUMERATOR(CopyCyanCompositeOp)
         ENUMERATOR(CopyMagentaCompositeOp)
         ENUMERATOR(CopyYellowCompositeOp)
         ENUMERATOR(CopyBlackCompositeOp)
-#endif
         ENUMERATOR(DarkenCompositeOp)
         ENUMERATOR(DifferenceCompositeOp)
         ENUMERATOR(DisplaceCompositeOp)
@@ -1437,6 +1569,9 @@ Init_RMagick(void)
         ENUMERATOR(ColorSeparationType)
         ENUMERATOR(ColorSeparationMatteType)
         ENUMERATOR(OptimizeType)
+#if defined(HAVE_PALETTEBILEVELMATTETYPE)
+        ENUMERATOR(PaletteBilevelMatteType)
+#endif
     END_ENUM
 
     // InterlaceType constants
@@ -1448,6 +1583,19 @@ Init_RMagick(void)
         ENUMERATOR(PartitionInterlace)
     END_ENUM
 
+#if defined(HAVE_INTERPOLATEPIXELCOLOR)
+    DEF_ENUM(InterpolatePixelMethod)
+        ENUMERATOR(UndefinedInterpolatePixel)
+        ENUMERATOR(AverageInterpolatePixel)
+        ENUMERATOR(BicubicInterpolatePixel)
+        ENUMERATOR(BilinearInterpolatePixel)
+        ENUMERATOR(FilterInterpolatePixel)
+        ENUMERATOR(IntegerInterpolatePixel)
+        ENUMERATOR(MeshInterpolatePixel)
+        ENUMERATOR(NearestNeighborInterpolatePixel)
+    END_ENUM
+#endif
+
 #if defined(HAVE_COMPAREIMAGELAYERS)
     DEF_ENUM(MagickLayerMethod)
         ENUMERATOR(UndefinedLayer)
@@ -1550,6 +1698,10 @@ Init_RMagick(void)
         ENUMERATOR(AndQuantumOperator)
         ENUMERATOR(DivideQuantumOperator)
         ENUMERATOR(LShiftQuantumOperator)
+#if defined(HAVE_MAXEVALUATEOPERATOR)
+        ENUMERATOR(MaxQuantumOperator)
+        ENUMERATOR(MinQuantumOperator)
+#endif
         ENUMERATOR(MultiplyQuantumOperator)
         ENUMERATOR(OrQuantumOperator)
         ENUMERATOR(RShiftQuantumOperator)
@@ -1752,9 +1904,8 @@ static void test_Magick_version(void)
         rb_raise(rb_eRuntimeError,
             "This version of RMagick was created to run with %s %s\n"
             "but %.*s is installed on this system. You should either\n"
-            "   1) refer to the RMagick README file to learn how to create\n"
-            "      a version of RMagick for %.*s, or\n"
-            "   2) download %s %s from %s and install it." ,
+            "   1) Configure and build RMagick for %.*s, or\n"
+            "   2) download %s %s from %s and install it.\n" ,
             MagickPackageName, MagickLibVersionText, x, version_str, x, version_str,
             MagickPackageName, MagickLibVersionText, web_site);
     }
@@ -1778,15 +1929,15 @@ static void version_constants(void)
     mgk_version = GetMagickVersion(NULL);
 
     str = rb_str_new2(mgk_version);
-    rb_obj_freeze(str);
+    (void) rb_obj_freeze(str);
     rb_define_const(Module_Magick, "Magick_version", str);
 
     str = rb_str_new2(PACKAGE_STRING);
-    rb_obj_freeze(str);
+    (void) rb_obj_freeze(str);
     rb_define_const(Module_Magick, "Version", str);
 
     sprintf(long_version,
-        "This is %s ($Date: 2006/06/02 23:26:53 $) Copyright (C) 2006 by Timothy P. Hunter\n"
+        "This is %s ($Date: 2007/06/09 16:45:25 $) Copyright (C) 2007 by Timothy P. Hunter\n"
         "Built with %s\n"
         "Built for %s\n"
         "Web page: http://rmagick.rubyforge.org\n"
@@ -1794,7 +1945,7 @@ static void version_constants(void)
         PACKAGE_STRING, mgk_version, RUBY_VERSION_STRING);
 
     str = rb_str_new2(long_version);
-    rb_obj_freeze(str);
+    (void) rb_obj_freeze(str);
     rb_define_const(Module_Magick, "Long_version", str);
 
 }
