@@ -1,6 +1,6 @@
-/* $Id: rmutil.c,v 1.76 2006/05/27 21:05:59 rmagick Exp $ */
+/* $Id: rmutil.c,v 1.90.2.3 2007/06/09 16:45:52 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2006 by Timothy P. Hunter
+|                Copyright (C) 2007 by Timothy P. Hunter
 | Name:     rmutil.c
 | Author:   Tim Hunter
 | Purpose:  Utility functions for RMagick
@@ -16,6 +16,50 @@ static void Color_Name_to_PixelPacket(PixelPacket *, VALUE);
 static VALUE Enum_type_values(VALUE);
 static VALUE Enum_type_inspect(VALUE);
 static void handle_exception(ExceptionInfo *, Image *, ErrorRetention);
+#if defined(HAVE_NEW_COLORINFO)
+static VALUE Pixel_from_MagickPixelPacket(MagickPixelPacket *);
+#endif
+
+#if !defined(HAVE_ACQUIREMAGICKMEMORY)
+/*
+    Dummy versions of ImageMagick memory routines for use with GraphicsMagick.
+*/
+static void *AcquireMagickMemory(const size_t size)
+{
+    assert(size > 0);
+    return malloc(size);
+}
+
+static void *RelinquishMagickMemory(void *memory)
+{
+    if (memory)
+    {
+        free(memory);
+    }
+    return NULL;
+}
+
+static void *ResizeMagickMemory(void *memory, const size_t size)
+{
+    void *new;
+
+    if (!memory)
+    {
+        return malloc(size);
+    }
+    if (size == 0)
+    {
+        free(memory);
+        return NULL;
+    }
+    new = realloc(memory, size);
+    if (!new)
+    {
+        free(memory);
+    }
+    return new;
+}
+#endif
 
 /*
     Extern:     magick_malloc, magick_free, magick_realloc
@@ -27,11 +71,7 @@ static void handle_exception(ExceptionInfo *, Image *, ErrorRetention);
 void *magick_malloc(const size_t size)
 {
     void *ptr;
-#if defined(HAVE_ACQUIREMAGICKMEMORY)
     ptr = AcquireMagickMemory(size);
-#else
-    ptr = AcquireMemory(size);
-#endif
     if (!ptr)
     {
         rb_raise(rb_eNoMemError, "not enough memory to continue");
@@ -42,23 +82,13 @@ void *magick_malloc(const size_t size)
 
 void magick_free(void *ptr)
 {
-#if defined(HAVE_ACQUIREMAGICKMEMORY)
-    RelinquishMagickMemory(ptr);
-#else
-    void *v = ptr;
-    LiberateMemory(&v);
-#endif
+    (void) RelinquishMagickMemory(ptr);
 }
 
 void *magick_realloc(void *ptr, const size_t size)
 {
     void *v;
-#if defined(HAVE_ACQUIREMAGICKMEMORY)
     v = ResizeMagickMemory(ptr, size);
-#else
-    v = ptr;
-    ReacquireMemory(&v, size);
-#endif
     if (!v)
     {
         rb_raise(rb_eNoMemError, "not enough memory to continue");
@@ -76,7 +106,7 @@ void *magick_realloc(void *ptr, const size_t size)
 */
 void magick_clone_string(char **new_str, const char *str)
 {
-    CloneString(new_str, str);
+    (void) CloneString(new_str, str);
 }
 
 
@@ -158,7 +188,7 @@ rm_strcasecmp(const char *s1, const char *s2)
  *  Purpose:    raise exception if array too short
 */
 void
-rm_check_ary_len(VALUE ary, int len)
+rm_check_ary_len(VALUE ary, long len)
 {
     if (RARRAY(ary)->len < len)
     {
@@ -234,7 +264,7 @@ rm_percentage(VALUE arg)
     int not_num;
 
     // Try to convert the argument to a number. If failure, sets not_num to non-zero.
-    rb_protect(arg_is_number, arg, &not_num);
+    (void) rb_protect(arg_is_number, arg, &not_num);
 
     if (not_num)
     {
@@ -284,7 +314,7 @@ rm_percentage(VALUE arg)
 static VALUE
 check_num2dbl(VALUE obj)
 {
-    rb_num2dbl(obj);
+    (void) rb_num2dbl(obj);
     return INT2FIX(1);
 }
 
@@ -357,7 +387,7 @@ rm_fuzz_to_dbl(VALUE fuzz_arg)
     int not_num;
 
     // Try to convert the argument to a number. If failure, sets not_num to non-zero.
-    rb_protect(arg_is_number, fuzz_arg, &not_num);
+    (void) rb_protect(arg_is_number, fuzz_arg, &not_num);
 
     if (not_num)
     {
@@ -407,7 +437,7 @@ rm_fuzz_to_dbl(VALUE fuzz_arg)
 VALUE
 ImageList_cur_image(VALUE img)
 {
-    return rb_funcall(img, ID_cur_image, 0);
+    return rb_funcall(img, rm_ID_cur_image, 0);
 }
 
 /*
@@ -459,7 +489,7 @@ Pixel_to_s(VALUE self)
 
     Data_Get_Struct(self, Pixel, pixel);
     sprintf(buff, "red=%d, green=%d, blue=%d, opacity=%d"
-          , pixel->red, pixel->green, pixel->blue, pixel->opacity);
+          , (int)pixel->red, (int)pixel->green, (int)pixel->blue, (int)pixel->opacity);
     return rb_str_new2(buff);
 }
 
@@ -477,14 +507,14 @@ Pixel_from_color(VALUE class, VALUE name)
 {
     PixelPacket pp;
     ExceptionInfo exception;
-    boolean okay;
+    MagickBooleanType okay;
 
     class = class;      // defeat "never referenced" message from icc
 
     GetExceptionInfo(&exception);
     okay = QueryColorDatabase(STRING_PTR(name), &pp, &exception);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     if (!okay)
     {
@@ -550,12 +580,12 @@ Pixel_to_color(int argc, VALUE *argv, VALUE self)
     image = AllocateImage(info);
     image->depth = depth;
     image->matte = matte;
-    DestroyImageInfo(info);
+    (void) DestroyImageInfo(info);
     GetExceptionInfo(&exception);
     (void) QueryColorname(image, pixel, compliance, name, &exception);
-    DestroyImage(image);
+    (void) DestroyImage(image);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     // Always return a string, even if it's ""
     return rb_str_new2(name);
@@ -574,8 +604,7 @@ Pixel_to_HSL(VALUE self)
     volatile VALUE hsl;
 
     Data_Get_Struct(self, Pixel, pixel);
-    TransformHSL(pixel->red, pixel->green, pixel->blue,
-                 &hue, &saturation, &luminosity);
+    TransformHSL(pixel->red, pixel->green, pixel->blue, &hue, &saturation, &luminosity);
 
     hsl = rb_ary_new3(3, rb_float_new(hue), rb_float_new(saturation),
                       rb_float_new(luminosity));
@@ -611,6 +640,16 @@ Pixel_from_HSL(VALUE class, VALUE hsl)
     return Pixel_from_PixelPacket(&rgb);
 }
 
+/*
+    Method:     Pixel#eql?
+    Purpose:    For use with Hash
+*/
+VALUE
+Pixel_eql_q(VALUE self, VALUE other)
+{
+    return NUM2INT(Pixel_spaceship(self, other)) == 0 ? Qtrue : Qfalse;
+}
+
 /*
     Method:  Pixel#fcmp(other[, fuzz[, colorspace]])
     Purpose: Compare pixel values for equality
@@ -660,7 +699,7 @@ Pixel_fcmp(int argc, VALUE *argv, VALUE self)
     {
         rb_raise(rb_eNoMemError, "not enough memory to continue");
     }
-    DestroyImageInfo(info);
+    (void) DestroyImageInfo(info);
 
     image->colorspace = colorspace;
     image->fuzz = fuzz;
@@ -670,7 +709,7 @@ Pixel_fcmp(int argc, VALUE *argv, VALUE self)
 #else
     equal = FuzzyColorCompare(image, this, that);
 #endif
-    DestroyImage(image);
+    (void) DestroyImage(image);
 
 #else
     equal = FuzzyColorMatch(this, that, fuzz);
@@ -679,6 +718,30 @@ Pixel_fcmp(int argc, VALUE *argv, VALUE self)
     return equal ? Qtrue : Qfalse;
 }
 
+
+/*
+    Method:     Pixel#hash
+    Notes:      INT2FIX left-shifts 1 bit. Sacrifice 1 bit
+                from the opacity attribute to the FIXNUM_FLAG.
+*/
+VALUE
+Pixel_hash(VALUE self)
+{
+    Pixel *pixel;
+    unsigned int hash;
+
+    Data_Get_Struct(self, Pixel, pixel);
+
+    hash  = ScaleQuantumToChar(pixel->red)   << 24;
+    hash += ScaleQuantumToChar(pixel->green) << 16;
+    hash += ScaleQuantumToChar(pixel->blue)  << 8;
+    hash += ScaleQuantumToChar(pixel->opacity);
+    hash >>= 1;
+
+    return INT2FIX(hash);
+
+}
+
 /*
     Method:  Pixel#intensity
     Purpose: Return the "intensity" of a pixel
@@ -687,14 +750,15 @@ VALUE
 Pixel_intensity(VALUE self)
 {
     Pixel *pixel;
-    unsigned long intensity;
+    Quantum intensity;
 
     Data_Get_Struct(self, Pixel, pixel);
 
-    intensity = (unsigned long)
-                (0.299*pixel->red) + (0.587*pixel->green) + (0.114*pixel->blue);
+    intensity = ROUND_TO_QUANTUM((0.299*pixel->red)
+                                + (0.587*pixel->green)
+                                + (0.114*pixel->blue));
 
-    return ULONG2NUM(intensity);
+    return ULONG2NUM((unsigned long) intensity);
 }
 
 
@@ -890,9 +954,9 @@ Pixel_dup(VALUE self)
     dup = Data_Wrap_Struct(CLASS_OF(self), NULL, destroy_Pixel, pixel);
     if (rb_obj_tainted(self))
     {
-        rb_obj_taint(dup);
+        (void) rb_obj_taint(dup);
     }
-    return rb_funcall(dup, ID_initialize_copy, 1, self);
+    return rb_funcall(dup, rm_ID_initialize_copy, 1, self);
 }
 
 
@@ -908,7 +972,7 @@ Pixel_clone(VALUE self)
     clone = Pixel_dup(self);
     if (OBJ_FROZEN(self))
     {
-        rb_obj_freeze(clone);
+        (void) rb_obj_freeze(clone);
     }
 
     return clone;
@@ -982,7 +1046,7 @@ PixelPacket_to_Color_Name(Image *image, PixelPacket *color)
 
     (void) QueryColorname(image, color, X11Compliance, name, &exception);
     CHECK_EXCEPTION()
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     return rb_str_new2(name);
 }
@@ -1009,12 +1073,12 @@ PixelPacket_to_Color_Name_Info(Info *info, PixelPacket *color)
     my_info = info ? info : CloneImageInfo(NULL);
 
     image = AllocateImage(info);
-    image->matte = False;
+    image->matte = MagickFalse;
     color_name = PixelPacket_to_Color_Name(image, color);
-    DestroyImage(image);
+    (void) DestroyImage(image);
     if (!info)
     {
-        DestroyImageInfo(my_info);
+        (void) DestroyImageInfo(my_info);
     }
 
     return color_name;
@@ -1028,20 +1092,21 @@ PixelPacket_to_Color_Name_Info(Info *info, PixelPacket *color)
 static void
 Color_Name_to_PixelPacket(PixelPacket *color, VALUE name_arg)
 {
-    boolean okay;
+    MagickBooleanType okay;
     char *name;
     ExceptionInfo exception;
 
     GetExceptionInfo(&exception);
     name = STRING_PTR(name_arg);
     okay = QueryColorDatabase(name, color, &exception);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
     if (!okay)
     {
         rb_raise(rb_eArgError, "invalid color name %s", name);
     }
 }
 
+
 /*
     Extern:     AffineMatrix_to_AffineMatrix
     Purpose:    Convert a Magick::AffineMatrix object to a AffineMatrix structure.
@@ -1057,7 +1122,7 @@ AffineMatrix_to_AffineMatrix(AffineMatrix *am, VALUE st)
         rb_raise(rb_eTypeError, "type mismatch: %s given",
                  rb_class2name(CLASS_OF(st)));
     }
-    values = rb_funcall(st, ID_values, 0);
+    values = rb_funcall(st, rm_ID_values, 0);
     v = rb_ary_entry(values, 0);
     am->sx = v == Qnil ? 1.0 : NUM2DBL(v);
     v = rb_ary_entry(values, 1);
@@ -1068,7 +1133,7 @@ AffineMatrix_to_AffineMatrix(AffineMatrix *am, VALUE st)
     am->sy = v == Qnil ? 1.0 : NUM2DBL(v);
     v = rb_ary_entry(values, 4);
     am->tx = v == Qnil ? 0.0 : NUM2DBL(v);
-    v = rb_ary_entry(values, 4);
+    v = rb_ary_entry(values, 5);
     am->ty = v == Qnil ? 0.0 : NUM2DBL(v);
 }
 
@@ -1254,12 +1319,10 @@ CompositeOperator_name(CompositeOperator op)
         ENUM_TO_NAME(ColorizeCompositeOp)
         ENUM_TO_NAME(CopyBlueCompositeOp)
         ENUM_TO_NAME(CopyCompositeOp)
-#if defined(HAVE_COPYCYANCOMPOSITEOP)   // CYMK added 5.5.7
         ENUM_TO_NAME(CopyCyanCompositeOp)
         ENUM_TO_NAME(CopyMagentaCompositeOp)
         ENUM_TO_NAME(CopyYellowCompositeOp)
         ENUM_TO_NAME(CopyBlackCompositeOp)
-#endif
         ENUM_TO_NAME(CopyGreenCompositeOp)
         ENUM_TO_NAME(CopyOpacityCompositeOp)
         ENUM_TO_NAME(CopyRedCompositeOp)
@@ -1483,6 +1546,9 @@ ImageType_name(ImageType type)
         ENUM_TO_NAME(ColorSeparationType)
         ENUM_TO_NAME(ColorSeparationMatteType)
         ENUM_TO_NAME(OptimizeType)
+#if defined(HAVE_PALETTEBILEVELMATTETYPE)
+        ENUM_TO_NAME(PaletteBilevelMatteType)
+#endif
     }
 
 }
@@ -1535,6 +1601,44 @@ InterlaceType_new(InterlaceType interlace)
 }
 
 
+/*
+    Static:     InterpolatePixelMethod_name
+    Purpose:    Return the name of a InterpolatePixelMethod enum as a string
+*/
+#if defined(HAVE_INTERPOLATEPIXELCOLOR)
+static const char *
+InterpolatePixelMethod_name(InterpolatePixelMethod interpolate)
+{
+    switch(interpolate)
+    {
+        default:
+        ENUM_TO_NAME(UndefinedInterpolatePixel)
+        ENUM_TO_NAME(AverageInterpolatePixel)
+        ENUM_TO_NAME(BicubicInterpolatePixel)
+        ENUM_TO_NAME(BilinearInterpolatePixel)
+        ENUM_TO_NAME(FilterInterpolatePixel)
+        ENUM_TO_NAME(IntegerInterpolatePixel)
+        ENUM_TO_NAME(MeshInterpolatePixel)
+        ENUM_TO_NAME(NearestNeighborInterpolatePixel)
+    }
+}
+
+
+/*
+    External:   InterpolatePixelMethod_new
+    Purpose:    Construct an InterpolatePixelMethod enum object for the specified value.
+*/
+VALUE
+InterpolatePixelMethod_new(InterpolatePixelMethod interpolate)
+{
+    const char *name;
+
+    name = InterpolatePixelMethod_name(interpolate);
+    return rm_enum_new(Class_InterpolatePixelMethod, ID2SYM(rb_intern(name)), INT2FIX(interpolate));
+}
+#endif
+
+
 /*
     External:   MagickLayerMethod_new
     Purpose:    Construct an MagickLayerMethod enum object for the specified value.
@@ -1693,9 +1797,13 @@ Color_from_ColorInfo(const ColorInfo *ci)
 
     compliance_type = ci->compliance;
     compliance = ComplianceType_new(compliance_type);
+#if defined(HAVE_NEW_COLORINFO)
+    color      = Pixel_from_MagickPixelPacket((MagickPixelPacket *)(&(ci->color)));
+#else
     color      = Pixel_from_PixelPacket((PixelPacket *)(&(ci->color)));
+#endif
 
-    return rb_funcall(Class_Color, ID_new, 3
+    return rb_funcall(Class_Color, rm_ID_new, 3
                     , name, compliance, color);
 }
 
@@ -1718,12 +1826,12 @@ Color_to_ColorInfo(ColorInfo *ci, VALUE st)
 
     memset(ci, '\0', sizeof(ColorInfo));
 
-    members = rb_funcall(st, ID_values, 0);
+    members = rb_funcall(st, rm_ID_values, 0);
 
     m = rb_ary_entry(members, 0);
     if (m != Qnil)
     {
-        CloneString((char **)&(ci->name), STRING_PTR(m));
+        (void) CloneString((char **)&(ci->name), STRING_PTR(m));
     }
     m = rb_ary_entry(members, 1);
     if (m != Qnil)
@@ -1734,7 +1842,18 @@ Color_to_ColorInfo(ColorInfo *ci, VALUE st)
     if (m != Qnil)
     {
         Data_Get_Struct(m, Pixel, pixel);
+#if defined(HAVE_NEW_COLORINFO)
+        // For >= 6.3.0, ColorInfo.color is a MagickPixelPacket so we have to
+        // convert the PixelPacket.
+        GetMagickPixelPacket(NULL, &ci->color);
+        ci->color.red = (MagickRealType) pixel->red;
+        ci->color.green = (MagickRealType) pixel->green;
+        ci->color.blue = (MagickRealType) pixel->blue;
+        ci->color.opacity = (MagickRealType) OpaqueOpacity;
+        ci->color.index = (MagickRealType) 0;
+#else
         ci->color = *pixel;
+#endif
     }
 }
 
@@ -1760,11 +1879,20 @@ Color_to_s(VALUE self)
     char buff[1024];
 
     Color_to_ColorInfo(&ci, self);
+
+#if defined(HAVE_NEW_COLORINFO)
+    sprintf(buff, "name=%s, compliance=%s, "
+                  "color.red=%g, color.green=%g, color.blue=%g, color.opacity=%g ",
+                  ci.name,
+                  ComplianceType_name(&ci.compliance),
+                  ci.color.red, ci.color.green, ci.color.blue, ci.color.opacity);
+#else
     sprintf(buff, "name=%s, compliance=%s, "
                   "color.red=%d, color.green=%d, color.blue=%d, color.opacity=%d ",
                   ci.name,
                   ComplianceType_name(&ci.compliance),
                   ci.color.red, ci.color.green, ci.color.blue, ci.color.opacity);
+#endif
 
     destroy_ColorInfo(&ci);
     return rb_str_new2(buff);
@@ -1786,6 +1914,28 @@ Pixel_from_PixelPacket(PixelPacket *pp)
 }
 
 
+/*
+    Static:     Pixel_from_MagickPixelPacket
+    Purpose:    Create a Magick::Pixel object from a MagickPixelPacket structure.
+    Notes:      bypasses normal Pixel.new, Pixel#initialize methods
+*/
+#if defined(HAVE_NEW_COLORINFO)
+static VALUE
+Pixel_from_MagickPixelPacket(MagickPixelPacket *pp)
+{
+    Pixel *pixel;
+
+    pixel          = ALLOC(Pixel);
+    pixel->red     = ROUND_TO_QUANTUM(pp->red);
+    pixel->green   = ROUND_TO_QUANTUM(pp->green);
+    pixel->blue    = ROUND_TO_QUANTUM(pp->blue);
+    pixel->opacity = ROUND_TO_QUANTUM(pp->opacity);
+
+    return Data_Wrap_Struct(Class_Pixel, NULL, destroy_Pixel, pixel);
+}
+#endif
+
+
 /*
  *  Static:     color_arg_rescue
  *  Purpose:    raise ArgumentError if the color name cannot be converted
@@ -1829,7 +1979,7 @@ Color_to_PixelPacket(PixelPacket *pp, VALUE color)
 VALUE
 PrimaryInfo_from_PrimaryInfo(PrimaryInfo *p)
 {
-    return rb_funcall(Class_Primary, ID_new, 3
+    return rb_funcall(Class_Primary, rm_ID_new, 3
                     , INT2FIX(p->x), INT2FIX(p->y), INT2FIX(p->z));
 }
 
@@ -1847,13 +1997,13 @@ PrimaryInfo_to_PrimaryInfo(PrimaryInfo *pi, VALUE sp)
         rb_raise(rb_eTypeError, "type mismatch: %s given",
                  rb_class2name(CLASS_OF(sp)));
     }
-    members = rb_funcall(sp, ID_values, 0);
+    members = rb_funcall(sp, rm_ID_values, 0);
     m = rb_ary_entry(members, 0);
-    pi->x = m == Qnil ? 0 : FIX2INT(m);
+    pi->x = m == Qnil ? 0.0 : NUM2DBL(m);
     m = rb_ary_entry(members, 1);
-    pi->y = m == Qnil ? 0 : FIX2INT(m);
+    pi->y = m == Qnil ? 0.0 : NUM2DBL(m);
     m = rb_ary_entry(members, 2);
-    pi->z = m == Qnil ? 0 : FIX2INT(m);
+    pi->z = m == Qnil ? 0.0 : NUM2DBL(m);
 }
 
 /*
@@ -1863,7 +2013,7 @@ PrimaryInfo_to_PrimaryInfo(PrimaryInfo *pi, VALUE sp)
 VALUE
 PointInfo_to_Point(PointInfo *p)
 {
-    return rb_funcall(Class_Point, ID_new, 2
+    return rb_funcall(Class_Point, rm_ID_new, 2
                     , INT2FIX(p->x), INT2FIX(p->y));
 }
 
@@ -1881,11 +2031,11 @@ Point_to_PointInfo(PointInfo *pi, VALUE sp)
         rb_raise(rb_eTypeError, "type mismatch: %s given",
                  rb_class2name(CLASS_OF(sp)));
     }
-    members = rb_funcall(sp, ID_values, 0);
+    members = rb_funcall(sp, rm_ID_values, 0);
     m = rb_ary_entry(members, 0);
-    pi->x = m == Qnil ? 0 : FIX2INT(m);
+    pi->x = m == Qnil ? 0.0 : NUM2DBL(m);
     m = rb_ary_entry(members, 1);
-    pi->y = m == Qnil ? 0 : FIX2INT(m);
+    pi->y = m == Qnil ? 0.0 : NUM2DBL(m);
 }
 
 
@@ -1908,7 +2058,7 @@ ChromaticityInfo_new(ChromaticityInfo *ci)
     blue_primary  = PrimaryInfo_from_PrimaryInfo(&ci->blue_primary);
     white_point   = PrimaryInfo_from_PrimaryInfo(&ci->white_point);
 
-    return rb_funcall(Class_Chromaticity, ID_new, 4
+    return rb_funcall(Class_Chromaticity, rm_ID_new, 4
                     , red_primary, green_primary, blue_primary, white_point);
 }
 
@@ -1930,7 +2080,7 @@ ChromaticityInfo_to_ChromaticityInfo(ChromaticityInfo *ci, VALUE chrom)
         rb_raise(rb_eTypeError, "type mismatch: %s given",
                  rb_class2name(CLASS_OF(chrom)));
     }
-    values_id = ID_values;
+    values_id = rm_ID_values;
 
     // Get the struct members in an array
     chrom_members = rb_funcall(chrom, values_id, 0);
@@ -1987,7 +2137,7 @@ Rectangle_from_RectangleInfo(RectangleInfo *rect)
     height = UINT2NUM(rect->height);
     x      = INT2NUM(rect->x);
     y      = INT2NUM(rect->y);
-    return rb_funcall(Class_Rectangle, ID_new, 4
+    return rb_funcall(Class_Rectangle, rm_ID_new, 4
                     , width, height, x, y);
 }
 
@@ -2005,7 +2155,7 @@ Rectangle_to_RectangleInfo(RectangleInfo *rect, VALUE sr)
         rb_raise(rb_eTypeError, "type mismatch: %s given",
                  rb_class2name(CLASS_OF(sr)));
     }
-    members = rb_funcall(sr, ID_values, 0);
+    members = rb_funcall(sr, rm_ID_values, 0);
     m = rb_ary_entry(members, 0);
     rect->width  = m == Qnil ? 0 : NUM2ULONG(m);
     m = rb_ary_entry(members, 1);
@@ -2029,7 +2179,7 @@ Segment_from_SegmentInfo(SegmentInfo *segment)
     y1 = rb_float_new(segment->y1);
     x2 = rb_float_new(segment->x2);
     y2 = rb_float_new(segment->y2);
-    return rb_funcall(Class_Segment, ID_new, 4, x1, y1, x2, y2);
+    return rb_funcall(Class_Segment, rm_ID_new, 4, x1, y1, x2, y2);
 }
 
 /*
@@ -2047,7 +2197,7 @@ Segment_to_SegmentInfo(SegmentInfo *segment, VALUE s)
                  rb_class2name(CLASS_OF(s)));
     }
 
-    members = rb_funcall(s, ID_values, 0);
+    members = rb_funcall(s, rm_ID_values, 0);
     m = rb_ary_entry(members, 0);
     segment->x1 = m == Qnil ? 0.0 : NUM2DBL(m);
     m = rb_ary_entry(members, 1);
@@ -2100,13 +2250,13 @@ Font_from_TypeInfo(TypeInfo *ti)
     family      = rb_str_new2(ti->family);
     style       = StyleType_new(ti->style);
     stretch     = StretchType_new(ti->stretch);
-    weight      = INT2NUM(ti->weight);
+    weight      = UINT2NUM(ti->weight);
     description = ti->description ? rb_str_new2(ti->description) : Qnil;
     encoding    = ti->encoding    ? rb_str_new2(ti->encoding) : Qnil;
     foundry     = ti->foundry     ? rb_str_new2(ti->foundry)  : Qnil;
     format      = ti->format      ? rb_str_new2(ti->format)   : Qnil;
 
-    return rb_funcall(Class_Font, ID_new, 9
+    return rb_funcall(Class_Font, rm_ID_new, 9
                     , name, description, family, style
                     , stretch, weight, encoding, foundry, format);
 }
@@ -2128,21 +2278,21 @@ Font_to_TypeInfo(TypeInfo *ti, VALUE st)
 
     memset(ti, '\0', sizeof(TypeInfo));
 
-    members = rb_funcall(st, ID_values, 0);
+    members = rb_funcall(st, rm_ID_values, 0);
     m = rb_ary_entry(members, 0);
     if (m != Qnil)
     {
-        CloneString((char **)&(ti->name), STRING_PTR(m));
+        (void) CloneString((char **)&(ti->name), STRING_PTR(m));
     }
     m = rb_ary_entry(members, 1);
     if (m != Qnil)
     {
-        CloneString((char **)&(ti->description), STRING_PTR(m));
+        (void) CloneString((char **)&(ti->description), STRING_PTR(m));
     }
     m = rb_ary_entry(members, 2);
     if (m != Qnil)
     {
-        CloneString((char **)&(ti->family), STRING_PTR(m));
+        (void) CloneString((char **)&(ti->family), STRING_PTR(m));
     }
     m = rb_ary_entry(members, 3); ti->style   = m == Qnil ? 0 : FIX2INT(m);
     m = rb_ary_entry(members, 4); ti->stretch = m == Qnil ? 0 : FIX2INT(m);
@@ -2150,13 +2300,13 @@ Font_to_TypeInfo(TypeInfo *ti, VALUE st)
 
     m = rb_ary_entry(members, 6);
     if (m != Qnil)
-        CloneString((char **)&(ti->encoding), STRING_PTR(m));
+        (void) CloneString((char **)&(ti->encoding), STRING_PTR(m));
     m = rb_ary_entry(members, 7);
     if (m != Qnil)
-        CloneString((char **)&(ti->foundry), STRING_PTR(m));
+        (void) CloneString((char **)&(ti->foundry), STRING_PTR(m));
     m = rb_ary_entry(members, 8);
     if (m != Qnil)
-        CloneString((char **)&(ti->format), STRING_PTR(m));
+        (void) CloneString((char **)&(ti->format), STRING_PTR(m));
 }
 
 
@@ -2247,7 +2397,7 @@ TypeMetric_from_TypeMetric(TypeMetric *tm)
     underline_position  = rb_float_new(tm->underline_position);
     underline_thickness = rb_float_new(tm->underline_position);
 
-    return rb_funcall(Class_TypeMetric, ID_new, 9
+    return rb_funcall(Class_TypeMetric, rm_ID_new, 9
                     , pixels_per_em, ascent, descent, width
                     , height, max_advance, bounds
                     , underline_position, underline_thickness);
@@ -2268,7 +2418,7 @@ TypeMetric_to_TypeMetric(TypeMetric *tm, VALUE st)
         rb_raise(rb_eTypeError, "type mismatch: %s given",
                  rb_class2name(CLASS_OF(st)));
     }
-    members = rb_funcall(st, ID_values, 0);
+    members = rb_funcall(st, rm_ID_values, 0);
 
     pixels_per_em   = rb_ary_entry(members, 0);
     Point_to_PointInfo(&tm->pixels_per_em, pixels_per_em);
@@ -2498,7 +2648,7 @@ VALUE Enum_spaceship(VALUE self, VALUE other)
 
     // Values are equal, check class.
 
-    return rb_funcall(CLASS_OF(self), ID_spaceship, 1, CLASS_OF(other));
+    return rb_funcall(CLASS_OF(self), rm_ID_spaceship, 1, CLASS_OF(other));
 }
 
 
@@ -2534,16 +2684,16 @@ VALUE Enum_type_initialize(VALUE self, VALUE sym, VALUE val)
 
     super_argv[0] = sym;
     super_argv[1] = val;
-    rb_call_super(2, (VALUE *)super_argv);
+    (void) rb_call_super(2, (VALUE *)super_argv);
 
-    if (rb_cvar_defined(CLASS_OF(self), ID_enumerators) != Qtrue)
+    if (rb_cvar_defined(CLASS_OF(self), rm_ID_enumerators) != Qtrue)
     {
-        RUBY18(rb_cvar_set(CLASS_OF(self), ID_enumerators, rb_ary_new(), 0));
-        RUBY16(rb_cvar_set(CLASS_OF(self), ID_enumerators, rb_ary_new()));
+        RUBY18(rb_cvar_set(CLASS_OF(self), rm_ID_enumerators, rb_ary_new(), 0));
+        RUBY16(rb_cvar_set(CLASS_OF(self), rm_ID_enumerators, rb_ary_new()));
     }
 
-    enumerators = rb_cvar_get(CLASS_OF(self), ID_enumerators);
-    rb_ary_push(enumerators, self);
+    enumerators = rb_cvar_get(CLASS_OF(self), rm_ID_enumerators);
+    (void) rb_ary_push(enumerators, self);
 
     return self;
 }
@@ -2576,13 +2726,13 @@ static VALUE Enum_type_values(VALUE class)
     volatile VALUE rv;
     int x;
 
-    enumerators = rb_cvar_get(class, ID_enumerators);
+    enumerators = rb_cvar_get(class, rm_ID_enumerators);
 
     if (rb_block_given_p())
     {
         for (x = 0; x < RARRAY(enumerators)->len; x++)
         {
-            rb_yield(rb_ary_entry(enumerators, x));
+            (void) rb_yield(rb_ary_entry(enumerators, x));
         }
         rv = class;
     }
@@ -2591,7 +2741,7 @@ static VALUE Enum_type_values(VALUE class)
         copy = rb_ary_new2(RARRAY(enumerators)->len);
         for (x = 0; x < RARRAY(enumerators)->len; x++)
         {
-            rb_ary_push(copy, rb_ary_entry(enumerators, x));
+            (void) rb_ary_push(copy, rb_ary_entry(enumerators, x));
         }
         OBJ_FREEZE(copy);
         rv = copy;
@@ -2661,30 +2811,30 @@ Statistics_new(ImageStatistics *stats)
     mean = rb_float_new(stats->red.mean);
     stddev = rb_float_new(stats->red.standard_deviation);
     var = rb_float_new(stats->red.variance);
-    red = rb_funcall(Class_StatisticsChannel, ID_new, 5, max, min, mean, stddev, var);
+    red = rb_funcall(Class_StatisticsChannel, rm_ID_new, 5, max, min, mean, stddev, var);
 
     min = rb_float_new(stats->green.minimum);
     max = rb_float_new(stats->green.maximum);
     mean = rb_float_new(stats->green.mean);
     stddev = rb_float_new(stats->green.standard_deviation);
     var = rb_float_new(stats->green.variance);
-    green = rb_funcall(Class_StatisticsChannel, ID_new, 5, max, min, mean, stddev, var);
+    green = rb_funcall(Class_StatisticsChannel, rm_ID_new, 5, max, min, mean, stddev, var);
 
     min = rb_float_new(stats->blue.minimum);
     max = rb_float_new(stats->blue.maximum);
     mean = rb_float_new(stats->blue.mean);
     stddev = rb_float_new(stats->blue.standard_deviation);
     var = rb_float_new(stats->blue.variance);
-    blue = rb_funcall(Class_StatisticsChannel, ID_new, 5, max, min, mean, stddev, var);
+    blue = rb_funcall(Class_StatisticsChannel, rm_ID_new, 5, max, min, mean, stddev, var);
 
     min = rb_float_new(stats->opacity.minimum);
     max = rb_float_new(stats->opacity.maximum);
     mean = rb_float_new(stats->opacity.mean);
     stddev = rb_float_new(stats->opacity.standard_deviation);
     var = rb_float_new(stats->opacity.variance);
-    opacity = rb_funcall(Class_StatisticsChannel, ID_new, 5, max, min, mean, stddev, var);
+    opacity = rb_funcall(Class_StatisticsChannel, rm_ID_new, 5, max, min, mean, stddev, var);
 
-    return rb_funcall(Class_Statistics, ID_new, 4, red, green, blue, opacity);
+    return rb_funcall(Class_Statistics, rm_ID_new, 4, red, green, blue, opacity);
 
 }
 #endif  // HAVE_GETIMAGESTATISTICS
@@ -2778,16 +2928,56 @@ StyleType_name(StyleType style)
 void
 rm_write_temp_image(Image *image, char *tmpnam)
 {
+
+#if defined(HAVE_SETIMAGEREGISTRY)
+    MagickBooleanType okay;
+    ExceptionInfo exception;
+    volatile VALUE id_value;
+    int id;
+
+    GetExceptionInfo(&exception);
+
+
+    // 'id' is always the value of its previous use
+    if (rb_cvar_defined(Module_Magick, rm_ID__tmpnam_) == Qtrue)
+    {
+        id_value = rb_cvar_get(Module_Magick, rm_ID__tmpnam_);
+        id = FIX2INT(id_value);
+    }
+    else
+    {
+        id = 0;
+        rb_define_class_variable(Module_Magick, "@@__tmpnam__", INT2FIX(id));
+    }
+
+    id += 1;
+    RUBY16(rb_cvar_set(Module_Magick, rm_ID__tmpnam_, INT2FIX(id));)
+    RUBY18(rb_cvar_set(Module_Magick, rm_ID__tmpnam_, INT2FIX(id), 0);)
+
+    sprintf(tmpnam, "mpri:%d", id);
+
+    // Omit "mpri:" from filename to form the key
+    okay = SetImageRegistry(ImageRegistryType, tmpnam+5, image, &exception);
+    CHECK_EXCEPTION()
+    DestroyExceptionInfo(&exception);
+    if (!okay)
+    {
+        rb_raise(rb_eRuntimeError, "SetImageRegistry failed.");
+    }
+
+#else
+
     long registry_id;
 
     registry_id = SetMagickRegistry(ImageRegistryType, image, sizeof(Image), &image->exception);
+    rm_check_image_exception(image, RetainOnError);
     if (registry_id < 0)
     {
         rb_raise(rb_eRuntimeError, "SetMagickRegistry failed.");
     }
-    rm_check_image_exception(image, RetainOnError);
-
     sprintf(tmpnam, "mpri:%ld", registry_id);
+#endif
+
 }
 
 /*
@@ -2826,23 +3016,23 @@ rm_not_implemented(void)
 }
 
 /*
-    Static:     raise_error(msg, loc)
+    Static:     rm_magick_error(msg, loc)
     Purpose:    create a new ImageMagickError object and raise an exception
     Notes:      does not return
                 This funky technique allows me to safely add additional
                 information to the ImageMagickError object in both 1.6.8 and
                 1.8.0. See www.ruby_talk.org/36408.
 */
-static void
-raise_error(const char *msg, const char *loc)
+void
+rm_magick_error(const char *msg, const char *loc)
 {
     volatile VALUE exc, mesg, extra;
 
     mesg = rb_str_new2(msg);
     extra = loc ? rb_str_new2(loc) : Qnil;
 
-    exc = rb_funcall(Class_ImageMagickError, ID_new, 2, mesg, extra);
-    rb_funcall(rb_cObject, rb_intern("raise"), 1, exc);
+    exc = rb_funcall(Class_ImageMagickError, rm_ID_new, 2, mesg, extra);
+    (void) rb_funcall(rb_cObject, rb_intern("raise"), 1, exc);
 }
 
 
@@ -2871,8 +3061,8 @@ ImageMagickError_initialize(int argc, VALUE *argv, VALUE self)
             rb_raise(rb_eArgError, "wrong number of arguments (%d for 0 to 2)", argc);
     }
 
-    rb_call_super(super_argc, (VALUE *)super_argv);
-    rb_iv_set(self, "@"MAGICK_LOC, extra);
+    (void) rb_call_super(super_argc, (VALUE *)super_argv);
+    (void) rb_iv_set(self, "@"MAGICK_LOC, extra);
 
 
     return self;
@@ -2894,13 +3084,13 @@ rm_get_geometry(
 {
     VALUE v;
 
-    v = rb_funcall(geom, ID_x, 0);
+    v = rb_funcall(geom, rm_ID_x, 0);
     *x = NUM2LONG(v);
-    v = rb_funcall(geom, ID_y, 0);
+    v = rb_funcall(geom, rm_ID_y, 0);
     *y = NUM2LONG(v);
-    v = rb_funcall(geom, ID_width, 0);
+    v = rb_funcall(geom, rm_ID_width, 0);
     *width = NUM2ULONG(v);
-    v = rb_funcall(geom, ID_height, 0);
+    v = rb_funcall(geom, rm_ID_height, 0);
     *height = NUM2ULONG(v);
 
     // Getting the flag field is a bit more difficult since it's
@@ -2911,10 +3101,10 @@ rm_get_geometry(
     {
         MagickEnum *magick_enum;
 
-        v = rb_funcall(geom, ID_flag, 0);
+        v = rb_funcall(geom, rm_ID_flag, 0);
         if (!Class_GeometryValue)
         {
-            Class_GeometryValue = rb_const_get(Module_Magick, ID_GeometryValue);
+            Class_GeometryValue = rb_const_get(Module_Magick, rm_ID_GeometryValue);
         }
         if (CLASS_OF(v) != Class_GeometryValue)
         {
@@ -2938,13 +3128,13 @@ Image *rm_clone_image(Image *image)
     ExceptionInfo exception;
 
     GetExceptionInfo(&exception);
-    clone = CloneImage(image, 0, 0, True, &exception);
+    clone = CloneImage(image, 0, 0, MagickTrue, &exception);
     if (!clone)
     {
         rb_raise(rb_eNoMemError, "not enough memory to continue");
     }
     rm_check_exception(&exception, clone, DestroyOnError);
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 
     return clone;
 }
@@ -2975,7 +3165,7 @@ MagickBooleanType rm_progress_monitor(
 
     method = rb_str_new2(rb_id2name(rb_frame_last_func()));
 
-    rval = rb_funcall((VALUE)client_data, ID_call, 3, method, offset, span);
+    rval = rb_funcall((VALUE)client_data, rm_ID_call, 3, method, offset, span);
 
     return RTEST(rval) ? MagickTrue : MagickFalse;
 }
@@ -3146,7 +3336,7 @@ rm_check_image_exception(Image *imglist, ErrorRetention retention)
         rm_check_exception(&exception, imglist, retention);
     }
 
-    DestroyExceptionInfo(&exception);
+    (void) DestroyExceptionInfo(&exception);
 }
 
 
@@ -3286,12 +3476,12 @@ handle_exception(ExceptionInfo *exception, Image *imglist, ErrorRetention retent
 
      extra[sizeof(extra)-1] = '\0';
 
-     DestroyExceptionInfo(exception);
-     raise_error(msg, extra);
+     (void) DestroyExceptionInfo(exception);
+     rm_magick_error(msg, extra);
 
 #else
-     DestroyExceptionInfo(exception);
-     raise_error(msg, NULL);
+     (void) DestroyExceptionInfo(exception);
+     rm_magick_error(msg, NULL);
 #endif
 
 }
